/**
 * 프로그램에서 공통으로 사용할 custom interface 를 정의 할때 여기서 정의 한다.
 * 기본 타입 설정이며, 서비스별 타입 설정은 MyInterfaces.ts 에서 하도록 한다.
 */
import { ReactNode } from 'react';
import type { MenuProps } from 'antd';
import { SUPPORT_LAYOUT_TYPE, SUPPORT_LOCALE, SUPPORT_THEME, MQTT_STATE, STATUS } from './DkValues';
/**
 * 일반적인 react component 를 만들때 확장의 base 로 사용할 property 정의
 */
export interface IMyProps {
    children?: ReactNode;
}
/**
 * theme 구조 정의.
 * theme 별로 순서대로 정의 하면 된다.
 * antd 를 사용하기 때문에, 특별한 경우가 아니면 antd 의 theme token 값을 가져와서 사용 하도록 한다.
 * antd theme token 에 사용 가능한 값은 일부를 정의 해 두었으며,
 *
 * https://ant.design/docs/react/customize-theme
 * https://ant.design/theme-editor
 *
 * import { theme } from 'antd';
 * console.log(theme);
 *
 * 위의 코드 결과(defaultConfig.token 에 들어 있는 항목)와 링크를 참고 하여 항목을 더 추가하여 사용 하도록 한다.
 * antd component 에서 색상이 꼬일 수 있으므로 component 는 별도로 색상을 주어야 한다.
 */
export interface ITheme {
    code: SUPPORT_THEME;
    antdTheme: {
        token: {
            colorBgBase: string;
            colorTextBase: string;
        };
        components?: {
            Alert?: {
                colorInfoBg?: string;
                colorText?: string;
            };
            Menu?: {
                itemBg?: string;
            };
        };
    };
    dkPalette: Array<string>;
    svcPalette: Array<string>;
}
/**
 * 로그인 이후, 얻을 수 있는 저장될 유저 정보.
 * 이 값으로 로그인 여부를 판단 할 수 있다.
 */
export interface IUser {
    /** 유저의 role id. 리소스 아이디 */
    roleId?: string;
    /** 로그인 할 때 얻는 접속 토큰. empty string 또는 undefined 이면 접속 토큰이 없음. */
    accessToken?: string;
    /** 접속 토큰 만료 시간. 0 보다 작거나 또는 undefined 이면 접속 토큰이 없거나, 이미 만료된 토큰 */
    accessTokenExpire?: number;
    /** 접속 토큰 발습 시간. */
    accessTokenIssue?: number;
    /** 유저 이름. empty string 또는 undefined 이면 유저 이름이 없음. */
    name?: string;
    /** 유저 아이디. empty string 또는 undefined 이면 유저 아이디가 없음. */
    userId?: string;
    /** 유저 별명. empty string 또는 undefined 이면 별명이 없음. */
    nickname?: string;
    /** 유저의 타임존. empty string 또는 undefined 이면 time-zone 을 알 수 없음. */
    timeZone?: string;
    /** 유저의 생년 월 일. yyyymmdd 포맷의 날짜. empty string 또는 undefined 이면 알 수 없음. */
    birth?: string;
}
/** 자체적으로 사용하는 sidebar menu 항목의 요소 구조 */
export interface ISidebarMenuItem {
    /** 메뉴의 아이디 */
    key: string;
    /** 메뉴명 */
    label: string;
    /** 메뉴명. label 과 동일한 값. antd tree 로 사용 하기 위해서 추가 함. */
    title: string;
    /** 아이콘명 */
    icon: string;
    /** 메뉴 순서 */
    order: number;
    /** 메뉴의 url. 클릭 했을 때 이동할 url. empty string 이면 url 없음 */
    urlpath: string;
    /** 페이지 구분명. 메뉴 트리에서 leaf 이 아니면 값이 없을 수 있다. 값이 없으면 empty string 으로 설정 된다. */
    pageid: string;
    /** 상위 메뉴의 아이디. empty string 이거나 undefined 라면 상위 메뉴 없음 */
    pid?: string;
    /** 로그인 이후 최초 보여줄 페이지 인지 여부 */
    isfirst?: string;
    /** url 변수에 사용될 키, 값 */
    params?: {
        [key: string]: string;
    };
    /** 활성 여부 */
    status?: STATUS;
}
/**
 * redux 에 저장되는 sidebar 정보
 */
export interface ISidebar {
    /** 사이드에 메뉴를 그리기 위해서 사용하는 메뉴 구조 항목 */
    menuTree: MenuProps['items'];
    /** tree 형태가 아닌 1열로 array 형태의 메뉴. 링크를 위해서 사용 */
    menuMap: {
        [menu_key: string]: ISidebarMenuItem;
    };
    /** 사이드바 메뉴의 최초 default 로 선택된 메뉴 아이템의 key */
    defaultSelectedKey: string;
    /** 사이드바 메뉴의 최초 default 로 open 된 메뉴 아이템의 key */
    defaultOpenKeys: Array<string>;
}
/** redux 에 저장되는 common component 정보 */
export interface IModalConfirmFunction {
    (): void;
}
export interface ICommonComponent {
    modal: {
        /** modal show/hide flag. true 일때 modal 을 보여 준다. */
        isShow: boolean;
        /** confirm 버튼을 클릭하고 처리 완료 했는지 여부. true 라면 클릭 이후, 아직 완료 되지 않음 */
        isConfirming: boolean;
        /** modal 의 타이틀 텍스트. intl 을 사용해서 보여 주기 때문에, 텍스트 아이디를 사용 할 수 있음. 값이 없으면 overall.modalTitle 를 사용 */
        title?: string;
        /** modal 의 본문 텍스트. intl 을 사용해서 보여 주기 때문에, 텍스트 아이디를 사용 할 수 있음. 값이 없으면 overall.modalBody 를 사용 */
        body?: string;
        /** ok 버튼의 텍스트. intl 을 사용해서 보여 주기 때문에, 텍스트 아이디를 사용 할 수 있음. 값이 없으면 overall.yes 를 사용 */
        okText?: string;
        /** cancel 버튼의 텍스트. intl 을 사용해서 보여 주기 때문에, 텍스트 아이디를 사용 할 수 있음. 값이 없으면 overall.no 를 사용 */
        cancelText?: string;
        /** modal ok(confirm) 버튼을 클릭 했을 때 호출할 함수 */
        fnConfirm: IModalConfirmFunction | null;
    };
}
/**
 * redux-thunk 로 api 호출시에 request 상태 정의.
 *
 * mqtt connection 에서도 사용 됨. mqtt connect 에서 사용될 때 다음과 같이 사용 된다.
 * - pending : mqtt broker 연결 중
 * - rejected : mqtt broker 연결 실패
 * - fulfilled : mqtt broker 연결 성공
 */
export interface IApiRequestState {
    pending?: boolean;
    rejected?: boolean;
    fulfilled?: boolean;
}
/**
 * 앱에서 mqtt broker 와 연결 할 때, mqtt broker 연결 정보
 */
export interface IMqtt {
    protocol: string;
    host: string;
    port: number;
    keepAlive: number;
    reconnectPeriod: number;
    userId: string;
    password: string;
    clientId: string;
    connectState?: MQTT_STATE;
}
/**
 * local storage 에 저장되는 mqtt 연결 환경
 */
export interface IMqttStorage extends IMqtt {
    subscribedTopics?: Array<string>;
}
/**
 * Overall 에 저장될 간단한 환경
 */
export interface ISimpleConfig {
    localeCode: SUPPORT_LOCALE;
    localeCodeISO: string;
    themeCode: string;
    layoutCode: SUPPORT_LAYOUT_TYPE;
    mqttAutoConnect: boolean;
}
/**
 * redux-saga 에서 mqtt channel 을 생성해서 사용 할때, channel 에서 되는 mqtt 데이터
 */
export interface IMqttChannelData {
    eventType: string;
    data?: any;
}
/** local storage 에 저장되는 api key, api secret 데이터 구조 */
export interface IApiKeySecret {
    apiKey: string;
    apiSecret: string;
}
/**
 * 로그인 이후, 정해진 페이지로 이동 하지 않고 이전에 보던 최종 페이지로 이동 하기 위해서
 * local storage 에 저장되는 최종 페이지 정보 구조
 */
export interface ILastPageInfo {
    /** url prefix 를 포함하는 최종 페이지 url path */
    urlPath: string;
    /** 데이터를 저장한 최종 시간. milli-sec 단위의 timestamp. */
    tm: number;
    /** @deprecated 페이지에 의해서 open 된 사이드바의 key id. 2023.10.13 이후 부터 사용 하지 않음 */
    sidebarOpenKeys?: Array<string>;
    /** @deprecated 페이지에 의해서 select 된 사이드바의 key id. 2023.10.13 이후 부터 사용 하지 않음 */
    sidebarSelectedKey?: string;
}
/**
 * breadcrumb 구조
 */
export interface IBreadcrumb {
    /** 보여주는 텍스트 */
    title: string;
    /**
     * 클릭 했을 때 이동할 페이지 url path. url prefix path 을 제외한 path
     * undefined 라면 클릭 반응 없음
     */
    href?: string;
    /** i18n text 인지 여부. true 일 경우는 i18n text 가 아님 */
    noneI18n?: boolean;
}
/** SearchTable 컴포넌트에서 보여지는 테이블 헤더 칼럼 구조 */
export interface ITableHeader {
    key: string;
    title: string;
    dataIndex: string;
    width?: number;
    render?: any;
    ellipsis?: boolean;
}
/** react timer 의 구조 */
export interface IReactTimerHookResult {
    totalSeconds: number;
    seconds: number;
    minutes: number;
    hours: number;
    days: number;
    isRunning: boolean;
    start: () => void;
    pause: () => void;
    resume: () => void;
    restart: (newExpiryTimestamp: Date, autoStart?: boolean) => void;
}
/** Select 에서 사용하는 option 구조 */
export interface ISelectOption {
    label: any;
    value: string | number;
}
