/**
 * 웹 storage 를 사용 할 수 있도록 지원하는 함수 모음
 */
declare class CkStorage {
    /**
     * local storage 또는 session storage 에서 데이터를 읽는다.
     * 만일 원하는 키로 저장된 값이 없다면 null 을 리턴 한다.
     *
     * @param skey {string} 저장된 데이터를 찾을 key
     * @param isSession {boolean | undefined} true 이면 session storage 를 사용 한다.
     * @returns {string | null} 저장된 값을 리턴 한다. 만일 값이 없다면 null 을 리턴 한다.
     * @private
     */
    private static read;
    /**
     * local storage 또는 session storage 에 데이터를 upsert 한다.
     * upsert 가 성공하면 true, 실패하면 false 를 리턴 한다.
     *
     * @param skey {string} 데이터를 저장할 값의 key
     * @param value {string | { [key: string]: any; }} 저장할 데이터
     * @param isSession {boolean | undefined} true 이면 session storage 를 사용 한다.
     * @returns {boolean} upsert 가 성공하면 true, 아니면 false 를 리턴 한다.
     * @private
     */
    private static upsert;
    /**
     * aes 값이 바뀌기 때문에, local storage 에 저장된 값을 모두 refresh 한다.
     *
     * @param prevAesKey {string} 이전에 aes 암호화 할때 사용한 aes 암호화 키
     * @param newAesKey {string} 다시 aes 암호화 할때 사용할 aes 암호화 키
     * @returns {void} 리턴되는 값 없음
     * @private
     */
    private static reloadSavedData;
    /**
     * aes 암화키가 바뀌면, local storage 에 저장된 데이터를 모두 새로 저장 한다.
     *
     * @returns {boolean} 성공하면 true, 실패하면 false 를 리턴 한다.
     */
    static initValue(): boolean;
    /**
     * 앱에서 필요한 값을 저장 한다.
     *
     * @param skey {string} 값의 키. isFirst 가 true 이면 사용하지 않기 때문에, empty string 값을 준다.
     * @param value {string | { [key: string]: any; }} 저장할 값. isFirst 가 true 이면 사용하지 않기 때문에, empty string 값을 준다.
     * @param isFirst {boolean | undefined} true 이면, aes 암호화 값을 저장 한다. true 가 아니면 aes 암호화키를 만들어서 저장 한다. initValue 함수에서 true 를 사용하기 때문에, 개발자가 임의로 true 를 사용하는 것은 없을 것 이다.
     * @param aesKey {string | undefined} 값을 주면 주어진 값을 aes key 값으로 사용하고, 값을 주지 않으면 저장 된 aes key 값을 읽어 사용 한다.
     * @param isSession {boolean | undefined} true 이면 session storage 를 사용 한다.
     * @returns {boolean} 성공하면 true 아니면 false 를 리턴 한다.
     */
    static save(skey: string, value: string | {
        [key: string]: any;
    }, isFirst?: boolean, aesKey?: string, isSession?: boolean): boolean;
    /**
     * local storage 나 session storage 에 저장된 데이터를 찾는다.
     * 암호화 되어 저장된 값은 read 함수를 사용하면 암호화된 값 그대로 리턴 하며, 본 함수는 암호화 까지 풀는 함수 이다.
     *
     * @param skey {string} storage 에서 찾을 데이터의 키
     * @param aesKey {string | undefined} 값을 주면 주어진 값을 aes key 값으로 사용하고, 값을 주지 않으면 저장 된 aes key 값을 읽어 사용 한다.
     * @param isSession {boolean | undefined} true 이면 session storage 를 사용 한다.
     * @returns {string | { [key: string]: any; } | null} 저장된 값이 없으면 null 을 리턴 한다.
     */
    static find(skey: string, aesKey?: string, isSession?: boolean): string | {
        [key: string]: any;
    } | null;
    /**
     * local storage 또는 session storage 에서 데이터를 삭제 한다.
     *
     * @param skey {string} 삭제할 데이터의 key
     * @param isSession {boolean | undefined} true 이면 session storage 를 사용 한다.
     * @returns {void} 리턴되는 값 없음
     */
    static remove(skey: string, isSession?: boolean): void;
}
export default CkStorage;
