/* eslint-disable import/no-extraneous-dependencies */
/**
 * 기본 값 설정이며, 서비스별 값 설정은 MyValues.ts 에서 하도록 한다.
 *
 * 프로그램에서 사용할 공통 값들을 여기에서 정의 한다.
 * 현재 한국어, 영어, 중국어 3개를 지원하도록 구성 되어 있으며, 만일 다른 locale 을 추가 해야 한다면 다음 작업을 해야 한다.
 *
 * - DkValues 파일에 dayjs import
 * - src/resource/i18n 에 언어 파일 추가
 * - DkValues.SUPPORT_LOCALE 에 언어 코드 추가
 * - PublicRouter.loadI18nResource 에 언어 코드에 맞는 리소스 파일 import 추가(import 내부에는 변수를 사용 불가 하기 때문에 별도로 몰아서 정의 불가능.)
 *
 * 테마가 추가 된다면, locale 과 같이 다음 작업을 한다.
 * - DkValues.SUPPORT_THEME 에 테마 코드 추가
 * - PublicRouter.loadThemeResource 에 theme code 에 맞는 리소스 파일 import 추가(import 내부에는 변수를 사용 불가 하기 때문에 별도로 몰아서 정의 불가능.)
 */
import 'dayjs/locale/ko';
import 'dayjs/locale/en';
import 'dayjs/locale/zh-cn';

//=====================================
// 기본으로 포함되는 값 정의
//=====================================

/** 성별 구분 */
const GENDER = {
    /** 남성 */
    MALE: 1,
    /** 여성 */
    FEMALE: 2,
    /** 알 수 없음 */
    UNKNOWN: 3,
    /** 알리고 싶지 않음 */
    NO_COMMENT: 4,
} as const;
type GENDER = typeof GENDER[keyof typeof GENDER];

/**
 * yes, no 구분
 */
const YES_OR_NO = {
    /** yes */
    YES: 1,
    /** no */
    NO: 2,
} as const;
type YES_OR_NO = typeof YES_OR_NO[keyof typeof YES_OR_NO];

/**
 * 데이터의 활성 상태 구분
 */
const STATUS = {
    /** 활성 */
    ENABLE: 1,
    /** 중지 */
    PAUSE: 2,
    /** 비활성 */
    DISABLE: 3,
} as const;
type STATUS = typeof STATUS[keyof typeof STATUS];

/**
 * 접근하는 클라이언트 타입
 */
const TERMINAL = {
    /** 웹 브라우저 */
    WEB: 1,
    /** 안드로이드 앱 */
    ANDROID: 2,
    /** ios 앱 */
    IOS: 3,
    /** 서버 */
    SERVER: 100,
} as const;
type TERMINAL = typeof TERMINAL[keyof typeof TERMINAL];

/**
 * 서버에 등록되는 간단한 환경 설정 종류
 */
const SIMPLE_CONFIG_TYPE = {
    /** reactjs 페이지명 */
    REACT_PAGE: {
        key: 1,
        text: 'setting.simpleConfig.reactPage',
    },
    /** 템플릿 그룹 */
    TEMPLATE_GROUP: {
        key: 2,
        text: 'setting.simpleConfig.templateGroup',
    },
    /** ANTD 아이콘 */
    ANTD_ICON: {
        key: 3,
        text: 'setting.simpleConfig.antdIcon',
    },
    /** 지원 언어 코드 */
    LANG_CODE: {
        key: 4,
        text: 'setting.simpleConfig.langCode',
    },
} as const;
type SIMPLE_CONFIG_TYPE = typeof SIMPLE_CONFIG_TYPE[keyof typeof SIMPLE_CONFIG_TYPE];

/** 지원하는 유저 Role. 서비스 마다 다를 수 있기 때문에, 서비스 별로 변경 해서 사용 해야 한다. */
const USER_ROLE: {
    [key: string]: {
        key: number;
        label: string;
    };
} = {
    /** root 관리자 */
    ROOT: {
        key: 1,
        label: 'overall.userRole.root',
    },
    /** 일반1 */
    NORMAL1: {
        key: 100,
        label: 'overall.userRole.normal1',
    },
} as const;
type USER_ROLE = typeof USER_ROLE[keyof typeof USER_ROLE];

/** 데이터 생성, 데이터 수정시 사용하는 drawer open 을 구별 */
const DRAWER_OPEN_TYPE = {
    /** 데이터 생성, 데이터 수정 구별 없이 open */
    OPEN: 1,
    /** 데이터 생성을 위해서 drawer open */
    OPEN_NEW: 2,
    /** 데이터 수정을 위해서 drawer open */
    OPEN_MOD: 3,
    /** drawer close */
    CLOSE: 100,
} as const;
type DRAWER_OPEN_TYPE = typeof DRAWER_OPEN_TYPE[keyof typeof DRAWER_OPEN_TYPE];

/** antd 테이블에서 보여줄 한 페이지에 보여줄 row 갯수를 선택 할 수 있는 목록 */
const TABLE_PAGE_SIZES = [15, 30, 50, 100];
if (process.env.NODE_ENV_TYPE) {
    // 개발자 개인 pc 환경이면 추가 된다.
    TABLE_PAGE_SIZES.unshift(2);
}

/**
 * 앱에서 지원하는 locale 설정. 지원하는 locale 이 추가되면 변경 되어야 한다.
 * 이것이 변경되면 LOCALE_RESOURCE 도 같이 변경 되어야 한다.
 */
const SUPPORT_LOCALE: { [key: string]: string; } = {
    // 한국어
    ko_KR: 'ko_KR',
    // 영어
    en_US: 'en_US',
    // 중국어
    zh_CN: 'zh_CN',
} as const;
type SUPPORT_LOCALE = typeof SUPPORT_LOCALE[keyof typeof SUPPORT_LOCALE];
const supportLocaleList: Array<string> = Object.keys(SUPPORT_LOCALE);
const SUPPORT_LOCALE_CODE_LIST: Array<string> = [];
for (const row of supportLocaleList) {
    SUPPORT_LOCALE_CODE_LIST.push(SUPPORT_LOCALE[row]);
}

/**
 * 지원 하는 테마 종류. 테마가 추가 되면 변경 되어야 한다.
 * themes 디렉토리에 지원하는 theme 의 값들이 정의 되어야 한다.
 */
const SUPPORT_THEME: { [key: string]: string; } = {
    CLASSIC: 'classic',
    DARK: 'dark',
} as const;
type SUPPORT_THEME = typeof SUPPORT_THEME[keyof typeof SUPPORT_THEME];
const supportThemeList: Array<string> = Object.keys(SUPPORT_THEME);
const SUPPORT_THEME_CODE_LIST: Array<string> = [];
for (const row of supportThemeList) {
    SUPPORT_THEME_CODE_LIST.push(SUPPORT_THEME[row]);
}

/**
 * 지원하는 layout 타입.
 * 항목은 layout 지원되는 종류에 따라 추가 하거나, 삭제 한다.
 */
const SUPPORT_LAYOUT_TYPE: { [key: string]: string; } = {
    A: 'a',
    B: 'b',
    C: 'c',
    D: 'd',
    E: 'e',
    F: 'f',
} as const;
type SUPPORT_LAYOUT_TYPE = typeof SUPPORT_LAYOUT_TYPE[keyof typeof SUPPORT_LAYOUT_TYPE];
const supportLayoutTypeList: Array<string> = Object.keys(SUPPORT_LAYOUT_TYPE);
const SUPPORT_LAYOUT_TYPE_LIST: Array<string> = [];
for (const row of supportLayoutTypeList) {
    SUPPORT_LAYOUT_TYPE_LIST.push(SUPPORT_LAYOUT_TYPE[row]);
}

/**
 * 리소스를 로딩하는 절차 정의
 */
const LOAD_STATE = {
    INIT: 1, // 최초 로딩 시도 조차 하지 않을때. 초기값
    LOADING: 2, // 로딩 시도
    COMPLETE: 3, // 로딩 완료
} as const;
type LOAD_STATE = typeof LOAD_STATE[keyof typeof LOAD_STATE];

/**
 * Redux-thunk 로 API 를 호출 할 때, API 호출 상태.
 *
 * mqtt broker 에 메시지를 publish, subscribe 할 때도 사용 한다.
 * - PENDING : publish/subscribe 대기
 * - FULFILLED : publish/subscribe 성공
 * - REJECTED : publish/subscribe 실패
 */
const API_REQUEST_STATE = {
    INIT: 1, // api request 하지 않음(초기 상태)
    PENDING: 2, // api request 이전(호출중 으로 판단 하면 됨)
    FULFILLED: 3, // api request 완료(성공으로 완료)
    REJECTED: 4, // api request 완료(실패로 완료)
} as const;
type API_REQUEST_STATE = typeof API_REQUEST_STATE[keyof typeof API_REQUEST_STATE];

/** http header value */
const HTTP_HEADER_VALUE = {
    JSON: 'application/json',
} as const;
type HTTP_HEADER_VALUE = typeof HTTP_HEADER_VALUE[keyof typeof HTTP_HEADER_VALUE];

/**
 * Rest API 호출 할 때, http method
 */
const HTTP_METHOD = {
    GET: 'GET',
    POST: 'POST',
    PUT: 'PUT',
    DELETE: 'DELETE',
} as const;
type HTTP_METHOD = typeof HTTP_METHOD[keyof typeof HTTP_METHOD];

/**
 * rest api 연결시에 사용될 고정값.
 */
const API_CONFIG = {
    KEY: process.env.REACT_APP_API_KEY || '',
    SECRET: process.env.REACT_APP_API_SECRET || '',
    HOST: process.env.REACT_APP_API_HOST || '',
    PORT: process.env.REACT_APP_API_PORT || '',
    TIMEOUT: Number(process.env.REACT_APP_API_TIMEOUT || '30000'),
} as const;
type API_CONFIG = typeof API_CONFIG[keyof typeof API_CONFIG];
const API_HOST = (API_CONFIG.PORT ? `${API_CONFIG.HOST}:${API_CONFIG.PORT}` : API_CONFIG.HOST);

const DKAPI_CORE_PREFIX = '/v1/admin';
/**
 * 기본 rest api 의 endpoint.
 * 서비스별 rest api 의 endpoint 는 MyValues.ts 에 정의 한다.
 */
const DKAPI_ENDPOINT = {
    /** 테이블 존재 및 데이터 생성 체크 */
    INIT_CHECK: `${DKAPI_CORE_PREFIX}/init/db/check`,
    /** 기본 테이블 생성 */
    INIT_TABLE: `${DKAPI_CORE_PREFIX}/init/db/table`,
    /** 기본 데이터 생성 */
    INIT_DATA: `${DKAPI_CORE_PREFIX}/init/db/data`,

    /** 접속 토큰 발급(로그인) - Open */
    LOGIN: `${DKAPI_CORE_PREFIX}/user/login`,
    /** 접속 토큰 폐기(로그 아웃) - Bearer */
    LOGOUT: `${DKAPI_CORE_PREFIX}/user/logout`,
    /** 접속 토큰 연장 - Bearer */
    ACCESS_TOKEN_CONTINUE: `${DKAPI_CORE_PREFIX}/user/access/token/continue`,

    /** Role 목록 - Bearer */
    ROLE_LIST: `${DKAPI_CORE_PREFIX}/role/list`,

    /** 메뉴 목록 - Bearer */
    MENU_LIST: `${DKAPI_CORE_PREFIX}/menu/list`,
    /** 메뉴 삭제(단일) - Bearer */
    MENU_DEL: `${DKAPI_CORE_PREFIX}/menu/remove`,
    /** 메뉴 추가 - Bearer */
    MENU_ADD: `${DKAPI_CORE_PREFIX}/menu`,
    /** 메뉴 수정 - Bearer */
    MENU_MOD: `${DKAPI_CORE_PREFIX}/menu`,
    /** 메뉴 상세 - Bearer */
    MENU_INFO: `${DKAPI_CORE_PREFIX}/menu`,
    /** 메뉴 순서 변경 - Bearer */
    MENU_CANGE_ORDER: `${DKAPI_CORE_PREFIX}/menu/lorder`,

    /** 간단한 환경 설정 목록 - Bearer */
    SIMPLE_CONFIG_LIST: `${DKAPI_CORE_PREFIX}/etc/config/list`,
    /** 간단한 환경 설정 삭제 - Bearer */
    SIMPLE_CONFIG_DEL: `${DKAPI_CORE_PREFIX}/etc/config/remove`,
    /** 간단한 환경 설정 추가 - Bearer */
    SIMPLE_CONFIG_ADD: `${DKAPI_CORE_PREFIX}/etc/config`,
    /** 간단한 환경 설정 수정 - Bearer */
    SIMPLE_CONFIG_MOD: `${DKAPI_CORE_PREFIX}/etc/config`,
    /** 간단한 환경 설정 조회 - Bearer */
    SIMPLE_CONFIG_INFO: `${DKAPI_CORE_PREFIX}/etc/config`,

    /** 로그인 허용/차단 IP 목록 - Bearer */
    CONFIRM_IP_LIST: `${DKAPI_CORE_PREFIX}/etc/allow/deny/cidr/list`,
    /** 로그인 허용/차단 IP 삭제 - Bearer */
    CONFIRM_IP_DEL: `${DKAPI_CORE_PREFIX}/etc/allow/deny/cidr/remove`,
    /** 로그인 허용/차단 IP 추가 - Bearer */
    CONFIRM_IP_ADD: `${DKAPI_CORE_PREFIX}/etc/allow/deny/cidr`,
    /** 로그인 허용/차단 IP 수정 - Bearer */
    CONFIRM_IP_MOD: `${DKAPI_CORE_PREFIX}/etc/allow/deny/cidr`,
    /** 로그인 허용/차단 IP 조회 - Bearer */
    CONFIRM_IP_INFO: `${DKAPI_CORE_PREFIX}/etc/allow/deny/cidr`,

    /** 템플릿 목록 - Bearer */
    TMPL_LIST: `${DKAPI_CORE_PREFIX}/template/list`,
    /** 템플릿 삭제 - Bearer */
    TMPL_DEL: `${DKAPI_CORE_PREFIX}/template/remove`,
    /** 템플릿 추가 - Bearer */
    TMPL_ADD: `${DKAPI_CORE_PREFIX}/template`,
    /** 템플릿 수정 - Bearer */
    TMPL_MOD: `${DKAPI_CORE_PREFIX}/template`,
    /** 템플릿 조회 - Bearer */
    TMPL_INFO: `${DKAPI_CORE_PREFIX}/template`,

    /** 파일 업로드 - Bearer */
    FILE_UPLOAD: `${DKAPI_CORE_PREFIX}/file/upload`,

    /** 게시물 목록 - Bearer */
    DOC_LIST: `${DKAPI_CORE_PREFIX}/board/document/list`,
    /** 게시물 조회 - Bearer */
    DOC_INFO: `${DKAPI_CORE_PREFIX}/board/document`,
    /** 게시물 생성 - Bearer */
    DOC_ADD: `${DKAPI_CORE_PREFIX}/board/document`,
    /** 게시물 수정 - Bearer */
    DOC_MOD: `${DKAPI_CORE_PREFIX}/board/document`,
    /** 게시물 목록 - Bearer */
    DOC_DELETE: `${DKAPI_CORE_PREFIX}/board/document/remove`,
} as const;
type DKAPI_ENDPOINT = typeof DKAPI_ENDPOINT[keyof typeof DKAPI_ENDPOINT];

/**
 * rest api 의 리턴 코드.
 * 기본으로 성공 코드는 정의 되어 있고, 에러 코드 구분이 필요하면 추가 하도록 한다.
 * 서비스별 에러 코드는 MyValues 에 등록 하도록 한다.
 */
const API_RETURN_CODE = {
    SUCCESS: 'S000001',
    INVALID_AUTH: 'E000004', // 인증 실패 - http code 401
} as const;
type API_RETURN_CODE = typeof API_RETURN_CODE[keyof typeof API_RETURN_CODE];

/**
 * mqtt broker 연결 사용시 broker 연결 정보
 */
const MQTT_CONFIG = {
    PROTOCOL: `${process.env.REACT_APP_MQTT_PROTOCOL}`,
    HOST: `${process.env.REACT_APP_MQTT_HOST}`,
    PORT: Number(process.env.REACT_APP_MQTT_PORT),
    KEEP_ALIVE: Number(process.env.REACT_APP_MQTT_KEEP_ALIVE),
    RECONNECT_PERIOD: Number(process.env.REACT_APP_MQTT_RECONNECT_PERIOD),
    QOS: Number(process.env.REACT_APP_MQTT_QOS),
    USER_ID: `${process.env.REACT_APP_MQTT_USER_ID}`,
    PASSWORD: `${process.env.REACT_APP_MQTT_PASSWORD}`,
    CLIENT_ID: `${process.env.REACT_APP_MQTT_CLIENT_ID}`,
    RECV_MAX_MSG_COUNT: Number(process.env.REACT_APP_MQTT_RECV_MAX_MSG_COUNT),
    RECV_DELAY: Number(process.env.REACT_APP_MQTT_RECV_DELAY),
    RECV_BUFFER_SIZE: Number(process.env.REACT_APP_MQTT_RECV_BUFFER_SIZE),
    AUTO_CONNECT: `${process.env.REACT_APP_MQTT_AUTO_CONNECT}`.toLowerCase() === 'true',
    AUTO_SUBSCRIBE_TOPICS: JSON.parse(`${process.env.REACT_APP_MQTT_AUTO_SUBSCRIBE_TOPICS}`),
} as const;
type MQTT_CONFIG = typeof MQTT_CONFIG[keyof typeof MQTT_CONFIG];

/**
 * mqtt broker 연결 상태
 * - INIT : mqtt broker 최초 연결 시도 이전. 초기값
 * - CONNECTING : 연결중
 * - RECONNECTING : 재 연결중
 * - CONNECTED : 연결 됨
 * - FAILED : 연결 실패
 * - DISCONNECT : 연결 종료. end 함수로 클라이언트가 연결 종료 시킴
 */
const MQTT_STATE = {
    INIT: 1,
    CONNECTING: 2,
    RECONNECTING: 3,
    CONNECTED: 4,
    FAILED: 5,
    DISCONNECT: 6, // reconnect 시에도 들어 올 수 있음.
    DISCONNECT_FORCE: 7, // 유저가 강제로 끊음.
} as const;
type MQTT_STATE = typeof MQTT_STATE[keyof typeof MQTT_STATE];

/**
 * dotenv 를 통해서 주입된 값
 */
const appVersion = (process.env.NODE_ENV_TYPE ? `${process.env.APP_VERSION}.${process.env.NODE_ENV}.${process.env.NODE_ENV_TYPE}` : `${process.env.APP_VERSION}.${process.env.NODE_ENV}`);
const DOTENV = {
    /** time zone */
    TIME_ZONE: `${process.env.REACT_APP_TIME_ZONE}`,
    /** url path 앞에 고정으로 붙을 prefix url */
    URL_PREFIX: `${process.env.REACT_APP_URL_PREFIX}`,
    /** 앱의 버전. package.json 의 version 값에 추가로 더 한 것 */
    VERSION: appVersion,
    /** 설치를 위해 테이블 및 기본 데이터 생성을 위한 api key */
    INIT_DB_DATA_AUTH_APP_KEY: `${process.env.REACT_APP_INIT_DB_DATA_AUTH_APP_KEY}`,
    /** 설치를 위해 테이블 및 기본 데이터 생성을 위한 api signature */
    INIT_DB_DATA_AUTH_SIGNATURE_KEY: `${process.env.REACT_APP_INIT_DB_DATA_AUTH_SIGNATURE_KEY}`,
    /** local storage 또는 session storage 에 저장되는 데이터를 암호화 하기 위한 aes key, iv 를 저장하는 키 */
    KEY_AES_SECRET: `${process.env.REACT_APP_KEY_AES_SECRET}`,
    /** Rest API 에 사용하는 api key, api secret 를 저장하는 키. 로그인 이후 api key, api secret 가 바뀌기 때문에 필요함. */
    KEY_API_KEY_SECRET: `${process.env.REACT_APP_KEY_API_KEY_SECRET}`,
    /** 로그인 유지를 위해, 유저 정보를 저장하는 키. access token 도 유저 정보에 포함되어 있음 */
    KEY_USER: `${process.env.REACT_APP_KEY_USER}`,
    /** locale code, theme code 등의 환경을 저장하는 키 */
    KEY_CONFIG: `${process.env.REACT_APP_KEY_CONFIG}`,
    /** mqtt broker 에 자동으로 연결 하기 위한 값을 저장하는 키 */
    KEY_MQTT: `${process.env.REACT_APP_KEY_MQTT}`,
    /** 사이드바에 출력되는 메뉴 목록을 저장하는 키. tree 형태 */
    KEY_SIDEBAR_MENU_TREE: `${process.env.REACT_APP_KEY_SIDEBAR_MENU_TREE}`,
    /** 사이드바에 출력되는 메뉴 목록을 저장하는 키. map 형태. 이동하기 위한 url path 를 구하기 위한 값 */
    KEY_SIDEBAR_MENU_MAP: `${process.env.REACT_APP_KEY_SIDEBAR_MENU_MAP}`,
    /** 최종 사용한 페이지 정보가 저장되는 local storage 에서 키 */
    KEY_LAST_PAGE: `${process.env.REACT_APP_KEY_LAST_PAGE}`,
    /** 기본으로 사용하는 DB 데이터가 없을때, 기본 데이터를 생성하는 페이지 url path. REACT_APP_URL_PREFIX 을 제외한 값*/
    URL_PATH_DB_DATA_INIT: `${process.env.REACT_APP_URL_PATH_DB_DATA_INIT}`,
    /** 로그인 페이지 url path. REACT_APP_URL_PREFIX 을 제외한 값 */
    URL_PATH_LOGIN: `${process.env.REACT_APP_URL_PATH_LOGIN}`,
    /** 인증이 필요한 페이지 접근시 인증 되지 않았 을 때 보여줄 페이지의 url path. REACT_APP_URL_PREFIX 을 제외한 값 */
    URL_PATH_NO_AUTH: `${process.env.REACT_APP_URL_PATH_NO_AUTH}`,
    /** 인증 실패 페이지에서 자동으로 로그인 페이지로 이동할 시간. 초 단위 */
    NO_AUTH_AUTO_SEC: Number(`${process.env.REACT_APP_NO_AUTH_AUTO_SEC}`),
    /** 파일 업로드시 사용하는 파일키 */
    FILE_KEY: `${process.env.REACT_APP_FILE_KEY}`,
} as const;
type DOTENV = typeof DOTENV[keyof typeof DOTENV];

/**
 * running 하는 것들에 대한 상태 정의
 */
const RUNNING_STATE = {
    INIT: 1, // 최초 초기 상태
    RUNNING: 2, // running 중
    STOP: 3, // stop 됨
} as const;
type RUNNING_STATE = typeof RUNNING_STATE[keyof typeof RUNNING_STATE];

/**
 * component > common > FormJSONEditor 가 사용 되는 위치 구분.
 * 위치에 따라 그려지는 FormJSONEditor 가 약간의 변화가 있다.
 */
const JSON_FORM_TYPE = {
    /** 일반적으로 사용되는 경우. editing 가능한 jsoneditor 를 표시. 기본값 */
    NORMAL_EDITOR: 1,
    /** 일반적으로 사용되는 경우. view mode 로 jsoneditor 를 표시 */
    NORMAL_VIEW: 2,
    /** 메뉴 관리 페이지에서 사용되는 경우 */
    MENU: 3,
    /** 템플릿 관리 페이지에서 사용되는 경우 */
    TEMPLATE: 4,
} as const;
type JSON_FORM_TYPE = typeof JSON_FORM_TYPE[keyof typeof JSON_FORM_TYPE];

/** 서버에서 받지 않은 페이지(사이드바 메뉴 트리에 등록되지 않는 페이지)를 breadcrumb 에 추가 하기 위해서 사용되는 메뉴 데이터 key */
const CUSTOM_BREADCRUMB_PAGE_KEY = '3061bf0e4b595c6a8618406b61737378';

const INIT_DB_CHECK_STATE = {
    PREV: 1,
    FAIL: 2,
} as const;
type INIT_DB_CHECK_STATE = typeof INIT_DB_CHECK_STATE[keyof typeof INIT_DB_CHECK_STATE];

/**
 * 모든 enum 값에 대해서 파라미터로 주어진 값이 정의된 값에 포함되어 있는지 체크 한다.
 * 다음과 같이 사용 한다.
 *
 * number 가 정의된 enum 일때는...
 * isValidValue<number>(TERMINAL, 1)
 *
 * string 이 정의된 enum 일때는...
 * isValidValue<string>(IMAGE_RESIZE_MODE, 'crop')
 * @param enumType enum 타입
 * @param checkValue 체크 할 값
 */
const isValidValue = <T>(enumType: { [name: string]: any; }, checkValue: T) => {
    const keys = Object.keys(enumType);
    for (let i = 0, len = keys.length; i < len; i += 1) {
        if (enumType[keys[i]] === checkValue) {
            return true;
        }
    }
    return false;
};

export {
    GENDER,
    YES_OR_NO,
    STATUS,
    TERMINAL,
    SIMPLE_CONFIG_TYPE,
    USER_ROLE,
    DRAWER_OPEN_TYPE,
    TABLE_PAGE_SIZES,
    SUPPORT_LOCALE,
    SUPPORT_LOCALE_CODE_LIST,
    SUPPORT_THEME,
    SUPPORT_THEME_CODE_LIST,
    SUPPORT_LAYOUT_TYPE,
    SUPPORT_LAYOUT_TYPE_LIST,
    LOAD_STATE,
    API_REQUEST_STATE,
    HTTP_METHOD,
    HTTP_HEADER_VALUE,
    API_CONFIG,
    API_HOST,
    DKAPI_ENDPOINT,
    API_RETURN_CODE,
    MQTT_CONFIG,
    MQTT_STATE,
    DOTENV,
    RUNNING_STATE,
    JSON_FORM_TYPE,
    INIT_DB_CHECK_STATE,
    CUSTOM_BREADCRUMB_PAGE_KEY,
    isValidValue,
};
