/* eslint-disable import/no-extraneous-dependencies,no-param-reassign */
import CryptoCore from 'crypto-js/core';
import MD5 from 'crypto-js/md5';
import Sha256 from 'crypto-js/sha256';
import HmacSHA256 from 'crypto-js/hmac-sha256';
import EncBase64 from 'crypto-js/enc-base64';
import EncUtf8 from 'crypto-js/enc-utf8';
import EncHex from 'crypto-js/enc-hex';
import AES from 'crypto-js/aes';
import PadPkcs7 from 'crypto-js/pad-pkcs7';
import { RawAxiosRequestConfig } from 'axios';
import type { IApiKeySecret } from '../DkInterfaces';
import {
    API_CONFIG,
    API_HOST,
    HTTP_METHOD,
    DOTENV,
} from '../DkValues';
import CkStorage from './CkStorage';

/**
 * hash 또는 암호화 함수 모음.
 * crypto-js 사용시 필요한 것만 import 해야 최종 결과물에 용량이 작아짐.
 */
class CkCrypto {
    /** local storage 에서 api key, api secret 를 조회 해서 값을 설정 했는지 여부 */
    private static READ_API_KEY_SCRET = false;

    /** local storage 에서 찾거나 환경 설정에서 읽은 rest api 용 api key */
    private static API_KEY = '';

    /** local storage 에서 찾거나 환경 설정에서 읽은 rest api 용 api secret */
    private static API_SECRET = '';

    /**
     * api key, api secret 를 local storage 또는 환경 설정에서 찾는다.
     * @private
     */
    private static findApiKeyAndSecret() {
        const apiKeySecret = <IApiKeySecret> CkStorage.find(DOTENV.KEY_API_KEY_SECRET);
        if (!apiKeySecret || !apiKeySecret.apiKey || !apiKeySecret.apiSecret) {
            CkCrypto.API_KEY = API_CONFIG.KEY;
            CkCrypto.API_SECRET = API_CONFIG.SECRET;
            return;
        }

        CkCrypto.READ_API_KEY_SCRET = true;
        CkCrypto.API_KEY = apiKeySecret.apiKey;
        CkCrypto.API_SECRET = apiKeySecret.apiSecret;
    }

    /**
     * ckstack 에서 사용하는 hmac-sha256 값을 구한다. Rest API 인증 처리시에 사용 됨.
     *
     * @param method {HTTP_METHOD} http request method
     * @param urlPath {string} api endpoint 의 url path. '?' 뒤에 붙어 있는 query string 은 제외 한다.
     * @param requestBody {string | { [key: string]: any; } | undefined} request 시에 body 에 들어가는 값. json object 나 string 둘 다 사용 가능.
     * @param contentType {string | undefined} http header 의 content-type 값. undefined 이면, 'application/json' 으로 자동 사용 된다.
     * @param date {string | undefined} http header 의 h-date 로 들어가는 값. 값이 없으면 현재 시간으로 설정 된다.
     * @param secret {string | undefined} signature 값을 만들기 위한 api secret 값. 값이 없으면 local storage 에 저장된 값이나, DkValues.API_CONFIG 에 설정된 값이 사용 된다.
     * @returns {{ signature: string; hdate: string; }} hmac-sha256 으로 만들어진 signature 와 signature 만들때 사용된 hdate 를 리턴 한다.
     * @private
     */
    private static hmacSha256Hash2(
        method: HTTP_METHOD,
        urlPath: string,
        requestBody?: string | { [key: string]: any; },
        contentType?: string,
        date?: string,
        secret?: string,
    ): { signature: string; hdate: string; } {
        // 기본 api key, api secret 를 준비 한다.
        if (!secret) {
            CkCrypto.findApiKeyAndSecret();
        }

        let md5Body = '';
        if (requestBody) {
            let rBody = '';
            if (typeof requestBody === 'object') {
                try {
                    rBody = JSON.stringify(requestBody);
                } catch (err) {
                    console.error(`[${this.name}]invalid request body json for hmac-sha256. [${requestBody}]`);
                }
            } else {
                rBody = requestBody;
            }

            if (rBody.length > 0) {
                md5Body = CkCrypto.md5Hash(rBody).toString();
            }
        }
        console.log(`[${this.name}] request md5 body [${md5Body}]`);

        const cType = (contentType === undefined ? 'application/json' : contentType);
        const hdate = date || (new Date()).toUTCString();
        const asecr = secret || CkCrypto.API_SECRET;
        const signaturePlain = `${method.toUpperCase()}\n${md5Body}\n${cType}\n${hdate}\n${urlPath}`;
        console.log(`[${this.name}] signature [${signaturePlain}], secret [${asecr}]`);

        const signatureSha = HmacSHA256(signaturePlain, asecr);
        // 서버에서 base64 로 결과를 내기 때문에, 동일하게 바꾼다.
        const signature = EncBase64.stringify(signatureSha);
        console.log(`[${this.name}] hmac-sha256 signature [${signature}]`);
        return { signature, hdate };
    }

    /**
     * ckstack 에서 사용하는 파일 업로드용 hmac-sha256 값을 구한다. Rest API 인증 처리시에 사용 됨.
     *
     * @param urlPath {string} api endpoint 의 url path. '?' 뒤에 붙어 있는 query string 은 제외 한다.
     * @param date {string | undefined} http header 의 h-date 로 들어가는 값. 값이 없으면 현재 시간으로 설정 된다.
     * @param secret {string | undefined} signature 값을 만들기 위한 api secret 값. 값이 없으면 local storage 에 저장된 값이나, DkValues.API_CONFIG 에 설정된 값이 사용 된다.
     * @returns {{ signature: string; hdate: string; }} hmac-sha256 으로 만들어진 signature 와 signature 만들때 사용된 hdate 를 리턴 한다.
     * @private
     */
    private static hmacSha256HashForFile2(
        urlPath: string,
        date?: string,
        secret?: string,
    ): { signature: string; hdate: string; } {
        // 기본 api key, api secret 를 준비 한다.
        if (!secret) {
            CkCrypto.findApiKeyAndSecret();
        }

        const md5Body = CkCrypto.md5Hash(DOTENV.FILE_KEY).toString();
        console.log(`[${this.name}] request md5 body [${md5Body}]`);

        const hdate = date || (new Date()).toUTCString();
        const asecr = secret || CkCrypto.API_SECRET;
        const signaturePlain = `${HTTP_METHOD.POST}\n${md5Body}\n${hdate}\n${urlPath}`;
        console.log(`[${this.name}] signature [${signaturePlain}], secret [${asecr}]`);

        const signatureSha = HmacSHA256(signaturePlain, asecr);
        // 서버에서 base64 로 결과를 내기 때문에, 동일하게 바꾼다.
        const signature = EncBase64.stringify(signatureSha);
        console.log(`[${this.name}] hmac-sha256 signature [${signature}]`);
        return { signature, hdate };
    }

    /**
     * open/bearer 타입의 rest api 호출을 위한 http header 값을 구한다.
     *
     * @param method {HTTP_METHOD} http request method
     * @param urlPath {string} api endpoint 의 url path. '?' 뒤에 붙어 있는 query string 은 제외 한다.
     * @param requestBody {string | { [key: string]: any; } | undefined} request 시에 body 에 들어가는 값. json object 나 string 둘 다 사용 가능.
     * @param acceptLang {string | undefined} http header 의 accept-lang 에 들어가는 값. 값이 없으면 'ko' 로 자동 설정 된다.
     * @param contentType {string | undefined} http header 의 content-type 값. undefined 이면, 'application/json' 으로 자동 사용 된다.
     * @param hDate {string | undefined} http header 의 h-date 로 들어가는 값. 값이 없으면 현재 시간으로 설정 된다.
     * @param apiKeyOrAccessToken {string | undefined} api secret 과 매핑 되는 api key 값. isBearer 가 true 라면 access token 값 이다.
     * @param apiSecret {string | undefined} signature 값을 만들기 위한 api secret 값. 값이 없으면 local storage 에 저장된 값이나, DkValues.API_CONFIG 에 설정된 값이 사용 된다.
     * @param isBearer {boolean | undefined} open 형태의 rest api header 를 만들지, bearer 형태의 rest api header 를 만들지 구분. true 이면 bearer 형태의 api header 를 만든다.
     * @returns {{ [key: string]: string; }} rest api 호출 할때 사용할 axios 용 기본 http header 를 만들어 리턴 한다.
     * @private
     */
    private static getAPIHeader(
        method: HTTP_METHOD,
        urlPath: string,
        requestBody?: string | { [key: string]: any; },
        acceptLang?: string,
        contentType?: string,
        hDate?: string,
        apiKeyOrAccessToken?: string,
        apiSecret?: string,
        isBearer?: boolean,
    ): { [key: string]: string; } {
        // 기본 api key, api secret 를 준비 한다.
        if (!apiSecret) {
            CkCrypto.findApiKeyAndSecret();
        }

        const sig = CkCrypto.hmacSha256Hash2(method, urlPath, requestBody, contentType, hDate, apiSecret);
        const aKey = apiKeyOrAccessToken || CkCrypto.API_KEY;
        const type = isBearer ? 'Bearer' : 'Open';
        const ret: { [key: string]: string; } = {
            Accept: 'application/json',
            'Accept-Language': acceptLang || 'ko',
            'H-Date': sig.hdate,
            'H-Authorization': `${type} ${aKey}:${sig.signature}`,
        };
        if (contentType) {
            ret['Content-Type'] = contentType;
        }
        return ret;
    }

    /**
     * open/bearer 타입의 파일 업로드를 위한 rest api 호출을 위한 http header 값을 구한다.
     *
     * @param urlPath {string} api endpoint 의 url path. '?' 뒤에 붙어 있는 query string 은 제외 한다.
     * @param acceptLang {string | undefined} http header 의 accept-lang 에 들어가는 값. 값이 없으면 'ko' 로 자동 설정 된다.
     * @param hDate {string | undefined} http header 의 h-date 로 들어가는 값. 값이 없으면 현재 시간으로 설정 된다.
     * @param apiKeyOrAccessToken {string | undefined} api secret 과 매핑 되는 api key 값. isBearer 가 true 라면 access token 값 이다.
     * @param apiSecret {string | undefined} signature 값을 만들기 위한 api secret 값. 값이 없으면 local storage 에 저장된 값이나, DkValues.API_CONFIG 에 설정된 값이 사용 된다.
     * @param isBearer {boolean | undefined} open 형태의 rest api header 를 만들지, bearer 형태의 rest api header 를 만들지 구분. true 이면 bearer 형태의 api header 를 만든다.
     * @returns {{ [key: string]: string; }} rest api 호출 할때 사용할 axios 용 기본 http header 를 만들어 리턴 한다.
     * @private
     */
    private static getAPIHeaderForFile(
        urlPath: string,
        acceptLang?: string,
        hDate?: string,
        apiKeyOrAccessToken?: string,
        apiSecret?: string,
        isBearer?: boolean,
    ): { [key: string]: string; } {
        // 기본 api key, api secret 를 준비 한다.
        if (!apiSecret) {
            CkCrypto.findApiKeyAndSecret();
        }

        const sig = CkCrypto.hmacSha256HashForFile2(urlPath, hDate, apiSecret);
        const aKey = apiKeyOrAccessToken || CkCrypto.API_KEY;
        const type = isBearer ? 'Bearer' : 'Open';
        const ret: { [key: string]: string; } = {
            Accept: 'application/json',
            'Accept-Language': acceptLang || 'ko',
            'H-Date': sig.hdate,
            'H-Authorization': `${type} ${aKey}:${sig.signature}`,
            'H-File-Key': DOTENV.FILE_KEY,
        };
        return ret;
    }

    /**
     * Buffer 를 crypto-js 에서 사용하는 word array 로 변경 한다.
     * 참고: https://gist.github.com/artjomb/7ef1ee574a411ba0dd1933c1ef4690d1
     *
     * @param buf {Buffer} word array 로 변경 할 Buffer 값
     * @returns {CryptoCore.lib.WordArray} crypto-js 에서 사용하는 word array 로 변경하여 리턴 한다.
     * @private
     */
    private static bufferToWordArray(buf: Buffer): CryptoCore.lib.WordArray {
        const wa: Array<any> = [];
        for (let i = 0, len = buf.length; i < len; i += 1) {
            wa[(i / 4) | 0] |= buf[i] << (24 - 8 * i);
        }

        return CryptoCore.lib.WordArray.create(wa, buf.length);
    }

    /**
     * crypto-js 의 word array 를 ArrayBuffer 로 변경 한다.
     * 참고1 : https://github.com/brix/crypto-js/issues/274
     * 참고2 : https://stackoverflow.com/questions/37228285/uint8array-to-arraybuffer
     *
     * @param wordArray {CryptoCore.lib.WordArray} 변경할 crypto-js word array
     * @return {ArrayBuffer} JS 의 array buffer 로 변경하여 리턴 한다.
     * @private
     */
    private static wordArrayToBuffer(wordArray: CryptoCore.lib.WordArray): ArrayBuffer {
        const l = wordArray.sigBytes;
        const { words } = wordArray;
        const result = new Uint8Array(l);

        let i = 0; // dst
        let j = 0; // src

        // eslint-disable-next-line no-constant-condition
        while (true) {
            // here i is a multiple of 4
            if (i === l) {
                break;
            }

            const w = words[j];
            j += 1;

            result[i] = (w & 0xff000000) >>> 24;
            i += 1;

            if (i === l) {
                break;
            }

            result[i] = (w & 0x00ff0000) >>> 16;
            i += 1;

            if (i === l) {
                break;
            }

            result[i] = (w & 0x0000ff00) >>> 8;
            i += 1;

            if (i === l) {
                break;
            }

            result[i] = (w & 0x000000ff);
            i += 1;
        }

        // return result;

        // 여기 까지 오면 result 에 Uint8Array 값이 설정 된다.
        return result.buffer.slice(result.byteOffset, result.byteLength + result.byteOffset);
    }

    /**
     * open 타입의 rest api 호출을 위한 axios option 값을 구한다.
     *
     * @param method {HTTP_METHOD} http request method
     * @param urlPath {string} api endpoint 의 url path. '?' 뒤에 붙어 있는 query string 은 제외 한다.
     * @param acceptLang {string} http header 의 accept-lang 에 들어가는 값. 특정 할 수 없으면 'en' 을 사용 한다.
     * @param requestBody {{ [key: string]: any; } | undefined} request 시에 body 에 들어가는 값. json object 나 string 둘 다 사용 가능.
     * @param qs {{ [key: string]: string; } | undefined} url 에 추가 되는 query string
     * @param contentType {string | undefined} http header 의 content-type 값. undefined 이면, 'application/json' 으로 자동 사용 된다.
     * @param hDate { string | undefined} http header 의 h-date 로 들어가는 값. 값이 없으면 현재 시간으로 설정 된다.
     * @param apiKey {string | undefined} api secret 과 매핑 되는 api key 값
     * @param apiSecret {string | undefined} signature 값을 만들기 위한 api secret 값. 값이 없으면 local storage 에 저장된 값이나, DkValues.API_CONFIG 에 설정된 값이 사용 된다.
     * @returns {RawAxiosRequestConfig} axios 를 사용하여 open 타입 rest api 호출 할때, axios(options) 으로 사용 할 수 있는 options 값을 리턴 한다.
     */
    public static getOpenAPIAxiosOptions(
        method: HTTP_METHOD,
        urlPath: string,
        acceptLang: string,
        requestBody?: { [key: string]: any; },
        qs?: { [key: string]: string; },
        contentType?: string,
        hDate?: string,
        apiKey?: string,
        apiSecret?: string,
    ): RawAxiosRequestConfig {
        const ret: RawAxiosRequestConfig = {
            method: method.toUpperCase(),
            url: `${API_HOST}${urlPath}`,
            headers: CkCrypto.getAPIHeader(method, urlPath, requestBody, acceptLang, contentType, hDate, apiKey, apiSecret),
            timeout: API_CONFIG.TIMEOUT,
        };
        if (requestBody) {
            ret.data = requestBody;
        }
        if (qs) {
            ret.params = qs;
        }
        return ret;
    }

    /**
     * bearer 타입의 rest api 호출을 위한 axios option 값을 구한다.
     *
     * @param method {HTTP_METHOD} http request method
     * @param urlPath {string} api endpoint 의 url path. '?' 뒤에 붙어 있는 query string 은 제외 한다.
     * @param acceptLang {string} http header 의 accept-lang 에 들어가는 값. 특정 할 수 없으면 'en' 을 사용 한다.
     * @param requestBody {{ [key: string]: any; } | undefined} request 시에 body 에 들어가는 값. json object 나 string 둘 다 사용 가능.
     * @param qs {{ [key: string]: string; } | undefined} url 에 추가 되는 query string
     * @param contentType {string | undefined} http header 의 content-type 값. undefined 이면, 'application/json' 으로 자동 사용 된다.
     * @param hDate {string | undefined} http header 의 h-date 로 들어가는 값. 값이 없으면 현재 시간으로 설정 된다.
     * @param accessToken {string | undefined} 접속 토큰
     * @param apiSecret {string | undefined} signature 값을 만들기 위한 api secret 값. 값이 없으면 local storage 에 저장된 값이나, DkValues.API_CONFIG 에 설정된 값이 사용 된다.
     * @returns {RawAxiosRequestConfig} axios 를 사용하여 bearer 타입 rest api 호출 할때, axios(options) 으로 사용 할 수 있는 options 값을 리턴 한다.
     */
    public static getBearerAPIAxiosOptions(
        method: HTTP_METHOD,
        urlPath: string,
        acceptLang: string,
        requestBody?: { [key: string]: any; },
        qs?: { [key: string]: string; },
        contentType?: string,
        hDate?: string,
        accessToken?: string,
        apiSecret?: string,
    ): RawAxiosRequestConfig {
        const ret: RawAxiosRequestConfig = {
            method: method.toUpperCase(),
            url: `${API_HOST}${urlPath}`,
            headers: CkCrypto.getAPIHeader(method, urlPath, requestBody, acceptLang, contentType, hDate, accessToken, apiSecret, true),
            timeout: API_CONFIG.TIMEOUT,
        };
        if (requestBody) {
            ret.data = requestBody;
        }
        if (qs) {
            ret.params = qs;
        }
        return ret;
    }

    public static getBearerAPIAxiosOptionsForFile(
        file: File,
        urlPath: string,
        acceptLang: string,
        qs?: { [key: string]: string; },
        hDate?: string,
        accessToken?: string,
        apiSecret?: string,
    ): RawAxiosRequestConfig {
        const formData = new FormData();
        formData.append(DOTENV.FILE_KEY, file);

        const ret: RawAxiosRequestConfig = {
            method: HTTP_METHOD.POST,
            url: `${API_HOST}${urlPath}`,
            headers: CkCrypto.getAPIHeaderForFile(urlPath, acceptLang, hDate, accessToken, apiSecret, true),
            timeout: API_CONFIG.TIMEOUT,
            data: formData,
        };
        if (qs) {
            ret.params = qs;
        }
        return ret;
    }

    /**
     * string 값을 md5 한다.
     *
     * @param value {string | Buffer} md5 할 원본값
     * @returns {string} md5 hash 값을 리턴 한다.
     */
    public static md5Hash(value: string | Buffer): string {
        if (typeof value === 'string') {
            return MD5(value).toString();
        }
        const wordArr = CkCrypto.bufferToWordArray(value);
        return MD5(wordArr).toString();
    }

    /**
     * string 값을 sha256 한다. 결과값으로 base64 로 인코딩 된 것을 리턴 한다.
     *
     * @param value {string} sha256 hash 할 값
     * @returns {string} base64 인코딩 된 값을 리턴 한다.
     */
    public static sha256Hash(value: string): string {
        return Sha256(value).toString(EncBase64);
    }

    /**
     * string 이나 buffer 를 base64 로 인코딩 한다.
     *
     * @param value {string | Buffer} 인코딩할 소스
     * @returns {string} base64 로 인코딩 하여 리턴 한다.
     */
    public static encodeBase64(value: string | Buffer): string {
        let wordArr;
        if (typeof value === 'string') {
            wordArr = CryptoCore.enc.Utf8.parse(value);
        } else {
            wordArr = CkCrypto.bufferToWordArray(value);
        }
        return EncBase64.stringify(wordArr);
    }

    /**
     * base64 로 인코딩 된 값을 디코딩 한다.
     *
     * @param value {string} base64 로 인코딩 된 값
     * @param isBuffer {boolean | undefined} 디코딩된 값이 string 이지 buffer 인지 구분. true 가 아니면 string 으로 취급 한다.
     * @returns {string | ArrayBuffer} 디코딩된 값을 리턴 한다.
     */
    public static decodeBase64(value: string, isBuffer?: boolean): string | ArrayBuffer {
        const wordArr = EncBase64.parse(value);
        if (isBuffer) {
            return CkCrypto.wordArrayToBuffer(wordArr);
        }

        return wordArr.toString(CryptoCore.enc.Utf8);
    }

    /**
     * string 이나 buffer 를 hex string 으로 인코딩 한다.
     *
     * @param value {string | Buffer} 인코딩할 소스
     * @returns {string} hex string 으로 인코딩 하여 리턴 한다.
     */
    public static encodeHex(value: string | Buffer): string {
        let wordArr;
        if (typeof value === 'string') {
            wordArr = CryptoCore.enc.Utf8.parse(value);
        } else {
            wordArr = CkCrypto.bufferToWordArray(value);
        }
        return CryptoCore.enc.Hex.stringify(wordArr);
    }

    /**
     * hex string 으로 인코딩 된 값을 디코딩 한다.
     *
     * @param value {string} hex string 으로 인코딩 된 값
     * @param isBuffer {boolean | undefined} 디코딩된 값이 string 이지 buffer 인지 구분. true 가 아니면 string 으로 취급 한다.
     * @returns {string | ArrayBuffer} 디코딩된 값을 리턴 한다.
     */
    public static decodeHex(value: string, isBuffer?: boolean): string | ArrayBuffer {
        const wordArr = CryptoCore.enc.Hex.parse(value);
        if (isBuffer) {
            return CkCrypto.wordArrayToBuffer(wordArr);
        }

        return CryptoCore.enc.Utf8.stringify(wordArr);
    }

    /**
     * string 값을 aes 암호화 한다.
     *
     * @param value {string} aes 암호화 할 값
     * @param secretKey {string} 암호화 할때 사용할 secret key
     * @param iv {string} 암호화 할때 사용할 iv
     * @param isByHex {boolean | undefined} 암호화 결과값을 hex 로 받을지 여부. true 이면 hex 형태의 string 으로 리턴하고, 아니면 crypto js utf8 형태의 string 으로 리턴 한다.
     * @returns {string} aes 암호화한 값을 리턴 한다. mode 는 cbc, 패팅 PadPkcs7 으로 암호화 한 것이고 파라미터의 isByHex 값에 따라 리턴되는 스트링을 base64 또는 hex 인코딩 할지 결정 된다.
     */
    public static encryptAES(value: string, secretKey: string, iv: string, isByHex?: boolean): string {
        const cipher = AES.encrypt(
            value,
            EncUtf8.parse(secretKey),
            {
                iv: EncUtf8.parse(iv),
                padding: PadPkcs7,
                mode: CryptoCore.mode.CBC,
            },
        );

        // return by hex
        if (isByHex) {
            const word = EncUtf8.parse(cipher.toString());
            return EncHex.stringify(word);
        }

        // return by base64
        return cipher.toString();
    }

    /**
     * aes 암호화 한 값을 푼다.
     *
     * @param value {string} aes 로 암호화 된 값
     * @param secretKey {string} 암호화를 풀기 위한 secret key
     * @param iv {string} 암호화를 풀기 위한 iv
     * @param isByHex {boolean | undefined} decrypt 할 값이 hex 형태 인지 아닌지 여부. true 이면 hex 형태. encryptAES 에서 isByHex 를 true 로 한 결과값이면 decrypt 시에도 isByHex 를 true 로 처리 해야 한다.
     * @returns {string} aes 로 암호화 된 값을 풀어서 리턴 한다. mode 는 cbc, 패팅 PadPkcs7 으로 aes 암호화 된 값을 푸는 것이고, isByHex 에 따라 input 되는 스트링이 hex 스트링인지, base64 스트링인지 구분 한다.
     */
    public static decryptAES(value: string, secretKey: string, iv: string, isByHex?: boolean): string {
        // hex 로 찍힌 것을
        if (isByHex) {
            const word = EncHex.parse(value);
            value = EncUtf8.stringify(word);
        }

        const decipher = AES.decrypt(
            value,
            EncUtf8.parse(secretKey),
            {
                iv: EncUtf8.parse(iv),
                padding: PadPkcs7,
                mode: CryptoCore.mode.CBC,
            },
        );
        return decipher.toString(EncUtf8);
    }
}

export default CkCrypto;
