"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var CkUtils_1 = __importDefault(require("./CkUtils"));
var CkCrypto_1 = __importDefault(require("./CkCrypto"));
var DkValues_1 = require("../DkValues");
/**
 * 웹 storage 를 사용 할 수 있도록 지원하는 함수 모음
 */
var CkStorage = /** @class */ (function () {
    function CkStorage() {
    }
    /**
     * local storage 또는 session storage 에서 데이터를 읽는다.
     * 만일 원하는 키로 저장된 값이 없다면 null 을 리턴 한다.
     *
     * @param skey {string} 저장된 데이터를 찾을 key
     * @param isSession {boolean | undefined} true 이면 session storage 를 사용 한다.
     * @returns {string | null} 저장된 값을 리턴 한다. 만일 값이 없다면 null 을 리턴 한다.
     * @private
     */
    CkStorage.read = function (skey, isSession) {
        if (!skey) {
            return null;
        }
        if (typeof window === 'undefined') {
            return null;
        }
        if (!window.localStorage || !window.sessionStorage) {
            console.warn("[".concat(this.name, "] cannot read storage value. not support localStorage, sessionStorage. need update web-browser version."));
            return null;
        }
        var value = (isSession ? window.sessionStorage.getItem(skey) : window.localStorage.getItem(skey));
        console.log("[".concat(this.name, "] read key [").concat(skey, "]"));
        return value;
    };
    /**
     * local storage 또는 session storage 에 데이터를 upsert 한다.
     * upsert 가 성공하면 true, 실패하면 false 를 리턴 한다.
     *
     * @param skey {string} 데이터를 저장할 값의 key
     * @param value {string | { [key: string]: any; }} 저장할 데이터
     * @param isSession {boolean | undefined} true 이면 session storage 를 사용 한다.
     * @returns {boolean} upsert 가 성공하면 true, 아니면 false 를 리턴 한다.
     * @private
     */
    CkStorage.upsert = function (skey, value, isSession) {
        if (!skey) {
            return false;
        }
        if (typeof window === 'undefined') {
            return false;
        }
        if (!window.localStorage || !window.sessionStorage) {
            console.warn("[".concat(this.name, "] cannot upsert data to storage. not support localStorage, sessionStorage. need update web-browser version."));
            return false;
        }
        var _value = (typeof value === 'object' ? JSON.stringify(value) : value);
        if (isSession) {
            window.sessionStorage.setItem(skey, _value);
        }
        else {
            window.localStorage.setItem(skey, _value);
        }
        console.log("[".concat(this.name, "] upsert. key [").concat(skey, "]"));
        return true;
    };
    /**
     * aes 값이 바뀌기 때문에, local storage 에 저장된 값을 모두 refresh 한다.
     *
     * @param prevAesKey {string} 이전에 aes 암호화 할때 사용한 aes 암호화 키
     * @param newAesKey {string} 다시 aes 암호화 할때 사용할 aes 암호화 키
     * @returns {void} 리턴되는 값 없음
     * @private
     */
    CkStorage.reloadSavedData = function (prevAesKey, newAesKey) {
        var refreshValue = function (skey) {
            var val = CkStorage.find(skey, prevAesKey);
            if (val === null) {
                return false;
            }
            return CkStorage.save(skey, val, false, newAesKey);
        };
        // note local storage 에 저장되는 데이터가 추가 된다면 여기에서 refresh 추가 해 주어야 한다.
        // 유저 데이터를 업데이트
        refreshValue(DkValues_1.DOTENV.KEY_USER);
        //  rest api 용 api key, api secret 업데이트
        refreshValue(DkValues_1.DOTENV.KEY_API_KEY_SECRET);
        // locale, theme 환경등을 가지고 있는 값을 업데이트
        refreshValue(DkValues_1.DOTENV.KEY_CONFIG);
        // mqtt 연결 정보를 가지고 있는 값을 업데이트
        refreshValue(DkValues_1.DOTENV.KEY_MQTT);
        // 사이드바 메뉴 정보를 업데이트
        refreshValue(DkValues_1.DOTENV.KEY_SIDEBAR_MENU_TREE);
        refreshValue(DkValues_1.DOTENV.KEY_SIDEBAR_MENU_MAP);
        // 최종 연결 페이지 정보를 업데이트
        refreshValue(DkValues_1.DOTENV.KEY_LAST_PAGE);
    };
    /**
     * aes 암화키가 바뀌면, local storage 에 저장된 데이터를 모두 새로 저장 한다.
     *
     * @returns {boolean} 성공하면 true, 실패하면 false 를 리턴 한다.
     */
    CkStorage.initValue = function () {
        var rPrevAesKey = CkStorage.read(DkValues_1.DOTENV.KEY_AES_SECRET);
        // 이전에 저장된 aes 암호화를 풀기 위한 값을 찾는다. 최초에는 저장된 값이 없다.
        // 암호화 처리를 위한 aes 값만 저장 하고 리턴 한다.
        if (!rPrevAesKey) {
            return CkStorage.save('', '', true);
        }
        var prevAesKey = rPrevAesKey.split('').reverse().join('');
        // 신규 aes 할 값을 만든다.
        CkStorage.save('', '', true);
        var raesKey = CkStorage.read(DkValues_1.DOTENV.KEY_AES_SECRET);
        if (!raesKey) {
            console.error("[".concat(this.name, "] not found secret value"));
            return false;
        }
        var currAesKey = raesKey.split('').reverse().join('');
        // 저장된 값들을 모두 새로 업데이트 한다.
        CkStorage.reloadSavedData(prevAesKey, currAesKey);
        return true;
    };
    /**
     * 앱에서 필요한 값을 저장 한다.
     *
     * @param skey {string} 값의 키. isFirst 가 true 이면 사용하지 않기 때문에, empty string 값을 준다.
     * @param value {string | { [key: string]: any; }} 저장할 값. isFirst 가 true 이면 사용하지 않기 때문에, empty string 값을 준다.
     * @param isFirst {boolean | undefined} true 이면, aes 암호화 값을 저장 한다. true 가 아니면 aes 암호화키를 만들어서 저장 한다. initValue 함수에서 true 를 사용하기 때문에, 개발자가 임의로 true 를 사용하는 것은 없을 것 이다.
     * @param aesKey {string | undefined} 값을 주면 주어진 값을 aes key 값으로 사용하고, 값을 주지 않으면 저장 된 aes key 값을 읽어 사용 한다.
     * @param isSession {boolean | undefined} true 이면 session storage 를 사용 한다.
     * @returns {boolean} 성공하면 true 아니면 false 를 리턴 한다.
     */
    CkStorage.save = function (skey, value, isFirst, aesKey, isSession) {
        // isFirst 가 true 이면 무조건 aes 암호화를 풀기 위한 암호키를 저장 한다.
        if (isFirst) {
            var uuid = CkUtils_1.default.getUUIDV5("".concat(CkUtils_1.default.getUUID(), "-").concat(Date.now())) + CkUtils_1.default.getUUIDV4() + CkUtils_1.default.getUUIDV4() + CkUtils_1.default.getUUIDV4();
            var ruuid = uuid.split('').reverse().join('');
            return CkStorage.upsert(DkValues_1.DOTENV.KEY_AES_SECRET, ruuid);
        }
        // aes 암호키를 찾는다.
        var _aesKey = aesKey;
        if (!_aesKey) {
            var raesKey = CkStorage.read(DkValues_1.DOTENV.KEY_AES_SECRET) || '';
            if (!raesKey) {
                console.error("[".concat(this.name, "] not found secret value"));
                return false;
            }
            _aesKey = raesKey.split('').reverse().join('');
        }
        // aes 암호화 하기 위한 secret key
        var aesSecret = CkCrypto_1.default.md5Hash(_aesKey);
        // aes 암호화 하기 위한 iv
        var iv = aesSecret.substring(0, 8) + aesSecret.substring(24);
        // 뒤집힌 원본 데이터를 암호화 하여 저장 할 것임
        var origVal = JSON.stringify(value).split('').reverse().join('');
        var encVal = CkCrypto_1.default.encryptAES(origVal, aesSecret, iv, true);
        return CkStorage.upsert(skey, encVal, isSession);
    };
    /**
     * local storage 나 session storage 에 저장된 데이터를 찾는다.
     * 암호화 되어 저장된 값은 read 함수를 사용하면 암호화된 값 그대로 리턴 하며, 본 함수는 암호화 까지 풀는 함수 이다.
     *
     * @param skey {string} storage 에서 찾을 데이터의 키
     * @param aesKey {string | undefined} 값을 주면 주어진 값을 aes key 값으로 사용하고, 값을 주지 않으면 저장 된 aes key 값을 읽어 사용 한다.
     * @param isSession {boolean | undefined} true 이면 session storage 를 사용 한다.
     * @returns {string | { [key: string]: any; } | null} 저장된 값이 없으면 null 을 리턴 한다.
     */
    CkStorage.find = function (skey, aesKey, isSession) {
        var encData = CkStorage.read(skey, isSession);
        if (!encData) {
            return null;
        }
        // 데이터는 aes 암호화 되어 있기 때문에 암호화 부터 해제 한다.
        var _aesKey = aesKey;
        if (!_aesKey) {
            var raesKey = CkStorage.read(DkValues_1.DOTENV.KEY_AES_SECRET);
            if (!raesKey) {
                console.error("[".concat(this.name, "] not found secret value"));
                return null;
            }
            _aesKey = raesKey.split('').reverse().join('');
        }
        // aes 풀기 위한 secret key
        var aesSecret = CkCrypto_1.default.md5Hash(_aesKey);
        // aes 풀기 위한 iv
        var iv = aesSecret.substring(0, 8) + aesSecret.substring(24);
        // 원본 데이터를 reverse 로 저장 했기 때문에, 다시 되돌린다.
        var decVal = '';
        try {
            decVal = CkCrypto_1.default.decryptAES(encData, aesSecret, iv, true).split('').reverse().join('');
        }
        catch (err) {
            console.log("[".concat(this.name, "] invalid value for decryptAES. skey [").concat(skey, "], encData [").concat(encData, "], aesSecret [").concat(aesSecret, "], iv [").concat(iv, "]"));
            return null;
        }
        return JSON.parse(decVal);
    };
    /**
     * local storage 또는 session storage 에서 데이터를 삭제 한다.
     *
     * @param skey {string} 삭제할 데이터의 key
     * @param isSession {boolean | undefined} true 이면 session storage 를 사용 한다.
     * @returns {void} 리턴되는 값 없음
     */
    CkStorage.remove = function (skey, isSession) {
        if (typeof window === 'undefined') {
            return;
        }
        if (!window.localStorage || !window.sessionStorage) {
            console.warn("[".concat(this.name, "] cannot delete data of storage. not support localStorage, sessionStorage. need update web-browser version."));
            return;
        }
        if (isSession === true) {
            window.sessionStorage.removeItem(skey);
        }
        else {
            window.localStorage.removeItem(skey);
        }
        console.log("[".concat(this.name, "] remove. key [").concat(skey, "]"));
    };
    return CkStorage;
}());
exports.default = CkStorage;
