/**
 * 기본 값 설정이며, 서비스별 값 설정은 MyValues.ts 에서 하도록 한다.
 *
 * 프로그램에서 사용할 공통 값들을 여기에서 정의 한다.
 * 현재 한국어, 영어, 중국어 3개를 지원하도록 구성 되어 있으며, 만일 다른 locale 을 추가 해야 한다면 다음 작업을 해야 한다.
 *
 * - DkValues 파일에 dayjs import
 * - src/resource/i18n 에 언어 파일 추가
 * - DkValues.SUPPORT_LOCALE 에 언어 코드 추가
 * - PublicRouter.loadI18nResource 에 언어 코드에 맞는 리소스 파일 import 추가(import 내부에는 변수를 사용 불가 하기 때문에 별도로 몰아서 정의 불가능.)
 *
 * 테마가 추가 된다면, locale 과 같이 다음 작업을 한다.
 * - DkValues.SUPPORT_THEME 에 테마 코드 추가
 * - PublicRouter.loadThemeResource 에 theme code 에 맞는 리소스 파일 import 추가(import 내부에는 변수를 사용 불가 하기 때문에 별도로 몰아서 정의 불가능.)
 */
import 'dayjs/locale/ko';
import 'dayjs/locale/en';
import 'dayjs/locale/zh-cn';
/** 성별 구분 */
declare const GENDER: {
    /** 남성 */
    readonly MALE: 1;
    /** 여성 */
    readonly FEMALE: 2;
    /** 알 수 없음 */
    readonly UNKNOWN: 3;
    /** 알리고 싶지 않음 */
    readonly NO_COMMENT: 4;
};
type GENDER = typeof GENDER[keyof typeof GENDER];
/**
 * yes, no 구분
 */
declare const YES_OR_NO: {
    /** yes */
    readonly YES: 1;
    /** no */
    readonly NO: 2;
};
type YES_OR_NO = typeof YES_OR_NO[keyof typeof YES_OR_NO];
/**
 * 데이터의 활성 상태 구분
 */
declare const STATUS: {
    /** 활성 */
    readonly ENABLE: 1;
    /** 중지 */
    readonly PAUSE: 2;
    /** 비활성 */
    readonly DISABLE: 3;
};
type STATUS = typeof STATUS[keyof typeof STATUS];
/**
 * 접근하는 클라이언트 타입
 */
declare const TERMINAL: {
    /** 웹 브라우저 */
    readonly WEB: 1;
    /** 안드로이드 앱 */
    readonly ANDROID: 2;
    /** ios 앱 */
    readonly IOS: 3;
    /** 서버 */
    readonly SERVER: 100;
};
type TERMINAL = typeof TERMINAL[keyof typeof TERMINAL];
/**
 * 서버에 등록되는 간단한 환경 설정 종류
 */
declare const SIMPLE_CONFIG_TYPE: {
    /** reactjs 페이지명 */
    readonly REACT_PAGE: {
        readonly key: 1;
        readonly text: "setting.simpleConfig.reactPage";
    };
    /** 템플릿 그룹 */
    readonly TEMPLATE_GROUP: {
        readonly key: 2;
        readonly text: "setting.simpleConfig.templateGroup";
    };
    /** ANTD 아이콘 */
    readonly ANTD_ICON: {
        readonly key: 3;
        readonly text: "setting.simpleConfig.antdIcon";
    };
    /** 지원 언어 코드 */
    readonly LANG_CODE: {
        readonly key: 4;
        readonly text: "setting.simpleConfig.langCode";
    };
};
type SIMPLE_CONFIG_TYPE = typeof SIMPLE_CONFIG_TYPE[keyof typeof SIMPLE_CONFIG_TYPE];
/** 지원하는 유저 Role. 서비스 마다 다를 수 있기 때문에, 서비스 별로 변경 해서 사용 해야 한다. */
declare const USER_ROLE: {
    [key: string]: {
        key: number;
        label: string;
    };
};
type USER_ROLE = typeof USER_ROLE[keyof typeof USER_ROLE];
/** 데이터 생성, 데이터 수정시 사용하는 drawer open 을 구별 */
declare const DRAWER_OPEN_TYPE: {
    /** 데이터 생성, 데이터 수정 구별 없이 open */
    readonly OPEN: 1;
    /** 데이터 생성을 위해서 drawer open */
    readonly OPEN_NEW: 2;
    /** 데이터 수정을 위해서 drawer open */
    readonly OPEN_MOD: 3;
    /** drawer close */
    readonly CLOSE: 100;
};
type DRAWER_OPEN_TYPE = typeof DRAWER_OPEN_TYPE[keyof typeof DRAWER_OPEN_TYPE];
/** antd 테이블에서 보여줄 한 페이지에 보여줄 row 갯수를 선택 할 수 있는 목록 */
declare const TABLE_PAGE_SIZES: number[];
/**
 * 앱에서 지원하는 locale 설정. 지원하는 locale 이 추가되면 변경 되어야 한다.
 * 이것이 변경되면 LOCALE_RESOURCE 도 같이 변경 되어야 한다.
 */
declare const SUPPORT_LOCALE: {
    [key: string]: string;
};
type SUPPORT_LOCALE = typeof SUPPORT_LOCALE[keyof typeof SUPPORT_LOCALE];
declare const SUPPORT_LOCALE_CODE_LIST: Array<string>;
/**
 * 지원 하는 테마 종류. 테마가 추가 되면 변경 되어야 한다.
 * themes 디렉토리에 지원하는 theme 의 값들이 정의 되어야 한다.
 */
declare const SUPPORT_THEME: {
    [key: string]: string;
};
type SUPPORT_THEME = typeof SUPPORT_THEME[keyof typeof SUPPORT_THEME];
declare const SUPPORT_THEME_CODE_LIST: Array<string>;
/**
 * 지원하는 layout 타입.
 * 항목은 layout 지원되는 종류에 따라 추가 하거나, 삭제 한다.
 */
declare const SUPPORT_LAYOUT_TYPE: {
    [key: string]: string;
};
type SUPPORT_LAYOUT_TYPE = typeof SUPPORT_LAYOUT_TYPE[keyof typeof SUPPORT_LAYOUT_TYPE];
declare const SUPPORT_LAYOUT_TYPE_LIST: Array<string>;
/**
 * 리소스를 로딩하는 절차 정의
 */
declare const LOAD_STATE: {
    readonly INIT: 1;
    readonly LOADING: 2;
    readonly COMPLETE: 3;
};
type LOAD_STATE = typeof LOAD_STATE[keyof typeof LOAD_STATE];
/**
 * Redux-thunk 로 API 를 호출 할 때, API 호출 상태.
 *
 * mqtt broker 에 메시지를 publish, subscribe 할 때도 사용 한다.
 * - PENDING : publish/subscribe 대기
 * - FULFILLED : publish/subscribe 성공
 * - REJECTED : publish/subscribe 실패
 */
declare const API_REQUEST_STATE: {
    readonly INIT: 1;
    readonly PENDING: 2;
    readonly FULFILLED: 3;
    readonly REJECTED: 4;
};
type API_REQUEST_STATE = typeof API_REQUEST_STATE[keyof typeof API_REQUEST_STATE];
/** http header value */
declare const HTTP_HEADER_VALUE: {
    readonly JSON: "application/json";
};
type HTTP_HEADER_VALUE = typeof HTTP_HEADER_VALUE[keyof typeof HTTP_HEADER_VALUE];
/**
 * Rest API 호출 할 때, http method
 */
declare const HTTP_METHOD: {
    readonly GET: "GET";
    readonly POST: "POST";
    readonly PUT: "PUT";
    readonly DELETE: "DELETE";
};
type HTTP_METHOD = typeof HTTP_METHOD[keyof typeof HTTP_METHOD];
/**
 * rest api 연결시에 사용될 고정값.
 */
declare const API_CONFIG: {
    readonly KEY: string;
    readonly SECRET: string;
    readonly HOST: string;
    readonly PORT: string;
    readonly TIMEOUT: number;
};
type API_CONFIG = typeof API_CONFIG[keyof typeof API_CONFIG];
declare const API_HOST: string;
/**
 * 기본 rest api 의 endpoint.
 * 서비스별 rest api 의 endpoint 는 MyValues.ts 에 정의 한다.
 */
declare const DKAPI_ENDPOINT: {
    /** 테이블 존재 및 데이터 생성 체크 */
    readonly INIT_CHECK: "/v1/admin/init/db/check";
    /** 기본 테이블 생성 */
    readonly INIT_TABLE: "/v1/admin/init/db/table";
    /** 기본 데이터 생성 */
    readonly INIT_DATA: "/v1/admin/init/db/data";
    /** 접속 토큰 발급(로그인) - Open */
    readonly LOGIN: "/v1/admin/user/login";
    /** 접속 토큰 폐기(로그 아웃) - Bearer */
    readonly LOGOUT: "/v1/admin/user/logout";
    /** 접속 토큰 연장 - Bearer */
    readonly ACCESS_TOKEN_CONTINUE: "/v1/admin/user/access/token/continue";
    /** Role 목록 - Bearer */
    readonly ROLE_LIST: "/v1/admin/role/list";
    /** 메뉴 목록 - Bearer */
    readonly MENU_LIST: "/v1/admin/menu/list";
    /** 메뉴 삭제(단일) - Bearer */
    readonly MENU_DEL: "/v1/admin/menu/remove";
    /** 메뉴 추가 - Bearer */
    readonly MENU_ADD: "/v1/admin/menu";
    /** 메뉴 수정 - Bearer */
    readonly MENU_MOD: "/v1/admin/menu";
    /** 메뉴 상세 - Bearer */
    readonly MENU_INFO: "/v1/admin/menu";
    /** 메뉴 순서 변경 - Bearer */
    readonly MENU_CANGE_ORDER: "/v1/admin/menu/lorder";
    /** 간단한 환경 설정 목록 - Bearer */
    readonly SIMPLE_CONFIG_LIST: "/v1/admin/etc/config/list";
    /** 간단한 환경 설정 삭제 - Bearer */
    readonly SIMPLE_CONFIG_DEL: "/v1/admin/etc/config/remove";
    /** 간단한 환경 설정 추가 - Bearer */
    readonly SIMPLE_CONFIG_ADD: "/v1/admin/etc/config";
    /** 간단한 환경 설정 수정 - Bearer */
    readonly SIMPLE_CONFIG_MOD: "/v1/admin/etc/config";
    /** 간단한 환경 설정 조회 - Bearer */
    readonly SIMPLE_CONFIG_INFO: "/v1/admin/etc/config";
    /** 로그인 허용/차단 IP 목록 - Bearer */
    readonly CONFIRM_IP_LIST: "/v1/admin/etc/allow/deny/cidr/list";
    /** 로그인 허용/차단 IP 삭제 - Bearer */
    readonly CONFIRM_IP_DEL: "/v1/admin/etc/allow/deny/cidr/remove";
    /** 로그인 허용/차단 IP 추가 - Bearer */
    readonly CONFIRM_IP_ADD: "/v1/admin/etc/allow/deny/cidr";
    /** 로그인 허용/차단 IP 수정 - Bearer */
    readonly CONFIRM_IP_MOD: "/v1/admin/etc/allow/deny/cidr";
    /** 로그인 허용/차단 IP 조회 - Bearer */
    readonly CONFIRM_IP_INFO: "/v1/admin/etc/allow/deny/cidr";
    /** 템플릿 목록 - Bearer */
    readonly TMPL_LIST: "/v1/admin/template/list";
    /** 템플릿 삭제 - Bearer */
    readonly TMPL_DEL: "/v1/admin/template/remove";
    /** 템플릿 추가 - Bearer */
    readonly TMPL_ADD: "/v1/admin/template";
    /** 템플릿 수정 - Bearer */
    readonly TMPL_MOD: "/v1/admin/template";
    /** 템플릿 조회 - Bearer */
    readonly TMPL_INFO: "/v1/admin/template";
    /** 파일 업로드 - Bearer */
    readonly FILE_UPLOAD: "/v1/admin/file/upload";
    /** 게시물 목록 - Bearer */
    readonly DOC_LIST: "/v1/admin/board/document/list";
    /** 게시물 조회 - Bearer */
    readonly DOC_INFO: "/v1/admin/board/document";
    /** 게시물 생성 - Bearer */
    readonly DOC_ADD: "/v1/admin/board/document";
    /** 게시물 수정 - Bearer */
    readonly DOC_MOD: "/v1/admin/board/document";
    /** 게시물 목록 - Bearer */
    readonly DOC_DELETE: "/v1/admin/board/document/remove";
};
type DKAPI_ENDPOINT = typeof DKAPI_ENDPOINT[keyof typeof DKAPI_ENDPOINT];
/**
 * rest api 의 리턴 코드.
 * 기본으로 성공 코드는 정의 되어 있고, 에러 코드 구분이 필요하면 추가 하도록 한다.
 * 서비스별 에러 코드는 MyValues 에 등록 하도록 한다.
 */
declare const API_RETURN_CODE: {
    readonly SUCCESS: "S000001";
    readonly INVALID_AUTH: "E000004";
};
type API_RETURN_CODE = typeof API_RETURN_CODE[keyof typeof API_RETURN_CODE];
/**
 * mqtt broker 연결 사용시 broker 연결 정보
 */
declare const MQTT_CONFIG: {
    readonly PROTOCOL: string;
    readonly HOST: string;
    readonly PORT: number;
    readonly KEEP_ALIVE: number;
    readonly RECONNECT_PERIOD: number;
    readonly QOS: number;
    readonly USER_ID: string;
    readonly PASSWORD: string;
    readonly CLIENT_ID: string;
    readonly RECV_MAX_MSG_COUNT: number;
    readonly RECV_DELAY: number;
    readonly RECV_BUFFER_SIZE: number;
    readonly AUTO_CONNECT: boolean;
    readonly AUTO_SUBSCRIBE_TOPICS: any;
};
type MQTT_CONFIG = typeof MQTT_CONFIG[keyof typeof MQTT_CONFIG];
/**
 * mqtt broker 연결 상태
 * - INIT : mqtt broker 최초 연결 시도 이전. 초기값
 * - CONNECTING : 연결중
 * - RECONNECTING : 재 연결중
 * - CONNECTED : 연결 됨
 * - FAILED : 연결 실패
 * - DISCONNECT : 연결 종료. end 함수로 클라이언트가 연결 종료 시킴
 */
declare const MQTT_STATE: {
    readonly INIT: 1;
    readonly CONNECTING: 2;
    readonly RECONNECTING: 3;
    readonly CONNECTED: 4;
    readonly FAILED: 5;
    readonly DISCONNECT: 6;
    readonly DISCONNECT_FORCE: 7;
};
type MQTT_STATE = typeof MQTT_STATE[keyof typeof MQTT_STATE];
declare const DOTENV: {
    /** time zone */
    readonly TIME_ZONE: string;
    /** url path 앞에 고정으로 붙을 prefix url */
    readonly URL_PREFIX: string;
    /** 앱의 버전. package.json 의 version 값에 추가로 더 한 것 */
    readonly VERSION: string;
    /** 설치를 위해 테이블 및 기본 데이터 생성을 위한 api key */
    readonly INIT_DB_DATA_AUTH_APP_KEY: string;
    /** 설치를 위해 테이블 및 기본 데이터 생성을 위한 api signature */
    readonly INIT_DB_DATA_AUTH_SIGNATURE_KEY: string;
    /** local storage 또는 session storage 에 저장되는 데이터를 암호화 하기 위한 aes key, iv 를 저장하는 키 */
    readonly KEY_AES_SECRET: string;
    /** Rest API 에 사용하는 api key, api secret 를 저장하는 키. 로그인 이후 api key, api secret 가 바뀌기 때문에 필요함. */
    readonly KEY_API_KEY_SECRET: string;
    /** 로그인 유지를 위해, 유저 정보를 저장하는 키. access token 도 유저 정보에 포함되어 있음 */
    readonly KEY_USER: string;
    /** locale code, theme code 등의 환경을 저장하는 키 */
    readonly KEY_CONFIG: string;
    /** mqtt broker 에 자동으로 연결 하기 위한 값을 저장하는 키 */
    readonly KEY_MQTT: string;
    /** 사이드바에 출력되는 메뉴 목록을 저장하는 키. tree 형태 */
    readonly KEY_SIDEBAR_MENU_TREE: string;
    /** 사이드바에 출력되는 메뉴 목록을 저장하는 키. map 형태. 이동하기 위한 url path 를 구하기 위한 값 */
    readonly KEY_SIDEBAR_MENU_MAP: string;
    /** 최종 사용한 페이지 정보가 저장되는 local storage 에서 키 */
    readonly KEY_LAST_PAGE: string;
    /** 기본으로 사용하는 DB 데이터가 없을때, 기본 데이터를 생성하는 페이지 url path. REACT_APP_URL_PREFIX 을 제외한 값*/
    readonly URL_PATH_DB_DATA_INIT: string;
    /** 로그인 페이지 url path. REACT_APP_URL_PREFIX 을 제외한 값 */
    readonly URL_PATH_LOGIN: string;
    /** 인증이 필요한 페이지 접근시 인증 되지 않았 을 때 보여줄 페이지의 url path. REACT_APP_URL_PREFIX 을 제외한 값 */
    readonly URL_PATH_NO_AUTH: string;
    /** 인증 실패 페이지에서 자동으로 로그인 페이지로 이동할 시간. 초 단위 */
    readonly NO_AUTH_AUTO_SEC: number;
    /** 파일 업로드시 사용하는 파일키 */
    readonly FILE_KEY: string;
};
type DOTENV = typeof DOTENV[keyof typeof DOTENV];
/**
 * running 하는 것들에 대한 상태 정의
 */
declare const RUNNING_STATE: {
    readonly INIT: 1;
    readonly RUNNING: 2;
    readonly STOP: 3;
};
type RUNNING_STATE = typeof RUNNING_STATE[keyof typeof RUNNING_STATE];
/**
 * component > common > FormJSONEditor 가 사용 되는 위치 구분.
 * 위치에 따라 그려지는 FormJSONEditor 가 약간의 변화가 있다.
 */
declare const JSON_FORM_TYPE: {
    /** 일반적으로 사용되는 경우. editing 가능한 jsoneditor 를 표시. 기본값 */
    readonly NORMAL_EDITOR: 1;
    /** 일반적으로 사용되는 경우. view mode 로 jsoneditor 를 표시 */
    readonly NORMAL_VIEW: 2;
    /** 메뉴 관리 페이지에서 사용되는 경우 */
    readonly MENU: 3;
    /** 템플릿 관리 페이지에서 사용되는 경우 */
    readonly TEMPLATE: 4;
};
type JSON_FORM_TYPE = typeof JSON_FORM_TYPE[keyof typeof JSON_FORM_TYPE];
/** 서버에서 받지 않은 페이지(사이드바 메뉴 트리에 등록되지 않는 페이지)를 breadcrumb 에 추가 하기 위해서 사용되는 메뉴 데이터 key */
declare const CUSTOM_BREADCRUMB_PAGE_KEY = "3061bf0e4b595c6a8618406b61737378";
declare const INIT_DB_CHECK_STATE: {
    readonly PREV: 1;
    readonly FAIL: 2;
};
type INIT_DB_CHECK_STATE = typeof INIT_DB_CHECK_STATE[keyof typeof INIT_DB_CHECK_STATE];
/**
 * 모든 enum 값에 대해서 파라미터로 주어진 값이 정의된 값에 포함되어 있는지 체크 한다.
 * 다음과 같이 사용 한다.
 *
 * number 가 정의된 enum 일때는...
 * isValidValue<number>(TERMINAL, 1)
 *
 * string 이 정의된 enum 일때는...
 * isValidValue<string>(IMAGE_RESIZE_MODE, 'crop')
 * @param enumType enum 타입
 * @param checkValue 체크 할 값
 */
declare const isValidValue: <T>(enumType: {
    [name: string]: any;
}, checkValue: T) => boolean;
export { GENDER, YES_OR_NO, STATUS, TERMINAL, SIMPLE_CONFIG_TYPE, USER_ROLE, DRAWER_OPEN_TYPE, TABLE_PAGE_SIZES, SUPPORT_LOCALE, SUPPORT_LOCALE_CODE_LIST, SUPPORT_THEME, SUPPORT_THEME_CODE_LIST, SUPPORT_LAYOUT_TYPE, SUPPORT_LAYOUT_TYPE_LIST, LOAD_STATE, API_REQUEST_STATE, HTTP_METHOD, HTTP_HEADER_VALUE, API_CONFIG, API_HOST, DKAPI_ENDPOINT, API_RETURN_CODE, MQTT_CONFIG, MQTT_STATE, DOTENV, RUNNING_STATE, JSON_FORM_TYPE, INIT_DB_CHECK_STATE, CUSTOM_BREADCRUMB_PAGE_KEY, isValidValue, };
