import { ISidebarMenuItem } from '../DkInterfaces';
import { IMenu } from '../DkApiInterfaces';
/**
 * 유틸리티 함수 모음
 */
declare class CkUtils {
    /**
     * uuid를 만든다. timestamp 값을 base 로 uuid 를 만든다.
     *
     * @param isContainDash {boolean | undefined} true 이면 dash 를 포함하는 36자리 uuid 를 만들고, true 가 아니면 32자리 uuid 를 만든다.
     * @returns {string} time base 로 만들어진 uuid 를 리턴 한다. v1 알고리즘으로 만든 uuid.
     */
    static getUUID(isContainDash?: boolean): string;
    /**
     * uuid를 만든다. random 값으로 uuid 를 만든다.
     *
     * @param isContainDash {boolean | undefined} true 이면 dash 를 포함하는 36자리 uuid 를 만들고, true 가 아니면 32자리 uuid 를 만든다.
     * @returns {string} 랜덤으로 만들어진 uuid 를 리턴 한다. v4 알고리즘으로 만든 uuid.
     */
    static getUUIDV4(isContainDash?: boolean): string;
    /**
     * uuid를 만든다. name, namespace 를 base 로 uuid 를 만든다.
     * nameSpace 는 dash 가 포함된 uuid 형태의 값을 사용 하면 된다.
     *
     * @param name {string} uuid 를 만들때 사용하는 name. 주로 CkUtils.getUUID() + '-' + Date.now() 로 사용 한다.
     * @param nameSpace {string | undefined} uuid 를 만들때 사용하는 name space. 값을 주지 않으면 내부에서 uuidv4 로 값을 생성 한다.
     * @param isContainDash {boolean | undefined} true 이면 dash 를 포함하는 36자리 uuid 를 만들고, true 가 아니면 32자리 uuid 를 만든다.
     * @returns {string} v5 알고리즘으로 만든 uuid 를 리턴 한다.
     */
    static getUUIDV5(name: string, nameSpace?: string, isContainDash?: boolean): string;
    /**
     * 설정된 url path prefix 가 포함된 url path 를 구한다.
     * url prefix 가 더해진 url path 를 만들어서 리턴 한다.
     *
     * @param path {string} prefix 가 붙지 않은 url path
     * @param params {{ [key: string]: string; } | undefined} url path 에 :param_list 등의 형태로 url path 가 들어가 있을 때, url path param 이 변경될 실제 값
     * @param isUpdateLastPage {boolean | undefined} 본 함수로 구하는 url path 를 last page 로 저장 할지 여부. true 일 경우 저장 한다.
     * @returns {string} url prefix 가 붙은 url path 를 리턴 한다.
     */
    static getSvcURLPath(path: string, params?: {
        [key: string]: string;
    }, isUpdateLastPage?: boolean): string;
    /**
     * URL 에 :param_offset 등이 있는 값 때문에, url path parameter 를 실제 값으로 변경 한다.
     *
     * @param urlPath {string | undefined} 값이 없으면 undefined 를 리턴 한다.
     * @param replaceValues {[key: string]: string;} replace 될 대상과 값. 키가 대상이고 값이 replace 된 값임
     * @returns {string | undefined} replace 된 url path 를 리턴 한다. 만일 urlPath 가 undefined 이면 undefined 를 리턴 한다.
     */
    static getReplacedURLPath(urlPath?: string, replaceValues?: {
        [key: string]: string;
    }): string | undefined;
    /**
     * cidr 의 허용하는 ip 범위를 구한다.
     *
     * 예1)
     * cidr 값이 39.118.41.11/32 이면
     * { startIP: '39.118.41.11', endIP: '39.118.41.11' } 가 리턴 된다. 시작과 끝이 같으므로 하나의 IP 만 가능
     *
     * 예2)
     * cidr 값이 39.118.41.11/30 이면
     * { startIP: '39.118.41.8', endIP: '39.118.41.11' } 가 리턴 된다. 8, 9, 10, 11 IP 만 가능
     *
     * 예3)
     * cidr 값이 39.118.41.11/23 이면
     * { startIP: '39.118.40.0', endIP: '39.118.41.255' } 가 리턴 된다.
     * 39.118.40.0 ~ 39.118.40.255, 39.118.41.0 ~ 39.118.41.255 IP 가능
     *
     * @param cidr {string} ip 허용 범위를 가져올 cidr 값
     * @param isPermitAsterica {boolean | undefined} cidr 에 asterica 를 사용 가능 한지 여부. cidr 이 asterica 라면 startIP, endIP 도 asterica 로 리턴 한다.
     * @returns {{ startIP: string; endIP: string; } | null} 허용하는 ip 범위를 리턴 한다. 범위를 구하지 못하면 null 을 리턴 한다.
     */
    static getApplyIpByCIDR(cidr: string, isPermitAsterica?: boolean): {
        startIP: string;
        endIP: string;
    } | null;
    /**
     * redux-saga 에서 rest api 호출시 axios catch(error) 에서 공통으로 사용하는 에러 response
     *
     * @param apiKind {string} 에러 로그에 포함되는 api 구분
     * @param thunkAPI {object} createAsyncThunk 함수의 thunkAPI
     * @param err {Error} axios 호출시 에러로 받은 error object
     * @returns {object} axios error 발생시 받은 response body 값. response body 가 없으면 null
     */
    static errorResponse(apiKind: string, thunkAPI: any, err?: any): any;
    /**
     * Rest API 로 받은 메뉴 데이터를 UI 에 사용 할 수 있는 형태로 파싱 한다.
     *
     * @param menus {IMenu[]} 서버에서 받은 menu 목록
     * @param withPageId {boolean | undefined} true 라면 메뉴명에 page id 도 포함 한다.
     * @returns {{ menuTree: Array<any>; menuMap: { [menu_key: string]: ISidebarMenuItem; } }} map 형태로 된 menu map 과 array 형태로된 menu tree 데이터를 리턴 한다.
     */
    static parsingMenu(menus: Array<IMenu>, withPageId?: boolean): {
        menuTree: Array<any>;
        menuMap: {
            [menu_key: string]: ISidebarMenuItem;
        };
    };
    /**
     * 서버에서 받은 지원하는 메뉴 목록에서 페이지 아이디에 맞는 메뉴 데이터를 찾는다. 만일 찾지 못하면 null 을 리턴 한다.
     *
     * @param pageId {string} 페이지 아이디
     * @param menuMap {{ [menu_key: string]: ISidebarMenuItem; })} 서버에서 받은 지원하는 전체 메뉴 데이터. redux Overall.sidebar.menuMap 데이터
     * @returns {ISidebarMenuItem | null} 데이터를 찾으면 메뉴 데이터를 리턴하고 찾지 못하면 null 을 리턴 한다.
     */
    static getPageMenuData(pageId: string, menuMap: {
        [menu_key: string]: ISidebarMenuItem;
    }): ISidebarMenuItem | undefined;
    /**
     * CkUtils.getPageMenuData 에서 구한 페이지의 메뉴 데이터에서 화면의 breadcrumb 에 표시할 데이터를 구한다.
     *
     * @param pageMenuData {ISidebarMenuItem} CkUtils.getPageMenuData 에서 구한 페이지의 메뉴 데이터
     * @param menuMap {{ [menu_key: string]: ISidebarMenuItem; })} 서버에서 받은 지원하는 전체 메뉴 데이터. redux Overall.sidebar.menuMap 데이터
     * @returns {ISidebarMenuItem[]} breadcrumb 에 표시할 메뉴 목록
     */
    static getBreadcrumbPath(pageMenuData: ISidebarMenuItem, menuMap: {
        [menu_key: string]: ISidebarMenuItem;
    }): Array<ISidebarMenuItem>;
    /**
     * CkUtils.getPageMenuData, CkUtils.getBreadcrumbPath 를 실행한 결과를 구한다. 페이지에서 동일한 패턴으로 사용하기 때문에 공통 처리를 추가함.
     *
     * @param pageId {string} 페이지 아이디
     * @param menuMap {{ [menu_key: string]: ISidebarMenuItem; })} 서버에서 받은 지원하는 전체 메뉴 데이터. redux Overall.sidebar.menuMap 데이터
     * @returns {{ menuData: ISidebarMenuItem | undefined; breadcrumb: Array<ISidebarMenuItem>; }} 페이지 메뉴 데이터와 breadcrumb 에서 사용할 메뉴 목록을 구한다.
     */
    static getPageMenuDataWithBreadcrumb(pageId: string, menuMap: {
        [menu_key: string]: ISidebarMenuItem;
    }): {
        menuData: ISidebarMenuItem | undefined;
        breadcrumb: Array<ISidebarMenuItem>;
    };
    /**
     * 숫자 형태의 소셜 구분값으로 부터 i18n text id 를 구한다.
     *
     * @param socialType {number} 소셜 구분
     * @returns {string} 소셜명 i18n text 를 리턴 한다.
     */
    static getSocialName(socialType: number): string;
    /**
     * 사이드바 메뉴 트리에 등록되지 않은 페이지의 메뉴 데이터를 만든다.
     * 서버에서 받지 않은 페이지를 breadcrumb 에 포함 시키기 위해서 사용 하는 함수 이다.
     *
     * @param pageTitle {string} 페이지 명
     * @param pageId {string | undefined} 페이지 아이디
     * @returns {ISidebarMenuItem} breadcrumb 에 추가할 임의의 페이지 데이터를 리턴 한다.
     */
    static getCustomBreadcrumbItem(pageTitle: string, pageId?: string): ISidebarMenuItem;
    /**
     * jsoneditor(https://github.com/josdejong/jsoneditor#code-mode) 를 위한 language code 를 변환 한다.
     * jsoneditor 는 기본으로 en, es zh-CN, pt-BR, tr, ja, fr-FR, de, ru, ko 언어 코드를 지원 한다.
     *
     * @param langCode {string} 변환할 언어 코드
     * @returns {string} 변환된 언어 코드
     */
    static getLocaleForJsonEditor(langCode: string): string;
}
export default CkUtils;
